@@ -25,7 +25,9 @@
 #include <kinputdialog.h>
 #include <kinstance.h>
 #include <klocale.h>
+#include <knotifyclient.h>
 #include <kprocess.h>
+#include <kpropertiesdialog.h>
 #include <kstatusbar.h>
 #include <kstdaction.h>
 #include <kxmlguifactory.h>
@@ -86,17 +88,17 @@ CervisiaPart::CervisiaPart( QWidget *parentWidget, const char *widgetName,
     , opt_doCVSEdit( false )
     , recent( 0 )
     , cvsService( 0 )
-#if KDE_IS_VERSION(3,1,90)
     , statusBar( 0 )
+    , m_browserExt( 0 )
     , filterLabel( 0 )
-#endif
     , m_editWithId(0)
     , m_currentEditMenu(0)
+    , m_jobType(Unknown)
 {
     KGlobal::locale()->insertCatalogue("cervisia");
 
     setInstance( CervisiaFactory::instance() );
-    new CervisiaBrowserExtension( this );
+    m_browserExt = new CervisiaBrowserExtension( this );
 
     // start the cvs DCOP service
     QString error;
@@ -139,11 +141,10 @@ CervisiaPart::CervisiaPart( QWidget *parentWidget, const char *widgetName,
         setWidget(splitter);
     }
     else
-        setWidget(new QLabel(i18n("This KPart is non-functional, because we "
-                                  "couldn't start the cvs DCOP service!"),
+        setWidget(new QLabel(i18n("This KPart is non-functional, because the "
+                                  "cvs DCOP service could not be started."),
                              parentWidget));
 
-#if KDE_IS_VERSION(3,1,90)
     statusBar = new CervisiaStatusBarExtension(this);
 
     // create the active filter indicator and add it to the statusbar
@@ -155,7 +156,6 @@ CervisiaPart::CervisiaPart( QWidget *parentWidget, const char *widgetName,
                        "N - All up-to-date files are hidden\n"
                        "R - All removed files are hidden"));
     statusBar->addStatusBarItem(filterLabel, 0, true);
-#endif
 
     if( cvsService )
     {
@@ -298,6 +298,11 @@ void CervisiaPart::setupActions()
     action->setToolTip( hint );
     action->setWhatsThis( hint );
 
+    // context menu only
+    action = new KAction( i18n("&Properties"), 0,
+                          this, SLOT( slotFileProperties() ),
+                          actionCollection(), "file_properties" );
+
     //
     // View Menu
     //
@@ -329,14 +334,14 @@ void CervisiaPart::setupActions()
     action->setToolTip( hint );
     action->setWhatsThis( hint );
 
-    action = new KAction( i18n("&Difference to Repository (BASE)..."), CTRL+Key_D,
+    action = new KAction( i18n("&Difference to Repository (BASE)..."), "vcs_diff", CTRL+Key_D,
                           this, SLOT(slotDiffBase()),
                           actionCollection(), "view_diff_base" );
     hint = i18n("Shows the differences of the selected file to the checked out version (tag BASE)");
     action->setToolTip( hint );
     action->setWhatsThis( hint );
 
-    action = new KAction( i18n("Difference to Repository (HEAD)..."), CTRL+Key_H,
+    action = new KAction( i18n("Difference to Repository (HEAD)..."), "vcs_diff", CTRL+Key_H,
                           this, SLOT(slotDiffHead()),
                           actionCollection(), "view_diff_head" );
     hint = i18n("Shows the differences of the selected file to the newest version in the repository (tag HEAD)");
@@ -507,9 +512,7 @@ void CervisiaPart::setupActions()
     KToggleAction* toggaction = new KToggleAction( i18n("Hide All &Files"), 0,
                                 this, SLOT(slotHideFiles()),
                                 actionCollection(), "settings_hide_files" );
-#if KDE_IS_VERSION(3,2,90)
     toggaction->setCheckedState(i18n("Show All &Files"));
-#endif
     hint = i18n("Determines whether only folders are shown");
     toggaction->setToolTip( hint );
     toggaction->setWhatsThis( hint );
@@ -517,9 +520,7 @@ void CervisiaPart::setupActions()
     toggaction = new KToggleAction( i18n("Hide Unmodified Files"), 0,
                                 this, SLOT(slotHideUpToDate()),
                                 actionCollection(), "settings_hide_uptodate" );
-#if KDE_IS_VERSION(3,2,90)
     toggaction->setCheckedState(i18n("Show Unmodified Files"));
-#endif
     hint = i18n("Determines whether files with status up-to-date or "
                 "unknown are hidden");
     toggaction->setToolTip( hint );
@@ -528,9 +529,7 @@ void CervisiaPart::setupActions()
     toggaction = new KToggleAction( i18n("Hide Removed Files"), 0,
                                 this, SLOT(slotHideRemoved()),
                                 actionCollection(), "settings_hide_removed" );
-#if KDE_IS_VERSION(3,2,90)
     toggaction->setCheckedState(i18n("Show Removed Files"));
-#endif
     hint = i18n("Determines whether removed files are hidden");
     toggaction->setToolTip( hint );
     toggaction->setWhatsThis( hint );
@@ -538,9 +537,7 @@ void CervisiaPart::setupActions()
     toggaction = new KToggleAction( i18n("Hide Non-CVS Files"), 0,
                                 this, SLOT(slotHideNotInCVS()),
                                 actionCollection(), "settings_hide_notincvs" );
-#if KDE_IS_VERSION(3,2,90)
     toggaction->setCheckedState(i18n("Show Non-CVS Files"));
-#endif
     hint = i18n("Determines whether files not in CVS are hidden");
     toggaction->setToolTip( hint );
     toggaction->setWhatsThis( hint );
@@ -548,9 +545,7 @@ void CervisiaPart::setupActions()
     toggaction = new KToggleAction( i18n("Hide Empty Folders"), 0,
                                     this, SLOT(slotHideEmptyDirectories()),
                                     actionCollection(), "settings_hide_empty_directories" );
-#if KDE_IS_VERSION(3,2,90)
     toggaction->setCheckedState(i18n("Show Empty Folders"));
-#endif
     hint = i18n("Determines whether folders without visible entries are hidden");
     toggaction->setToolTip( hint );
     toggaction->setWhatsThis( hint );
@@ -620,7 +615,7 @@ void CervisiaPart::popupRequested(KListView*, QListViewItem*, const QPoint& p)
     if (QPopupMenu* popup = static_cast<QPopupMenu*>(hostContainer("context_popup")))
     {
         // remove old 'Edit with...' menu
-        if( m_editWithId )
+        if( m_editWithId && popup->findItem(m_editWithId) != 0 )
         {
             popup->removeItem(m_editWithId);
             delete m_currentEditMenu; 
@@ -660,6 +655,8 @@ void CervisiaPart::updateActions()
     stateChanged("has_single_selection", single ? StateNoReverse
                                                 : StateReverse);
 
+    m_browserExt->setPropertiesActionEnabled(single);
+
     //    bool nojob = !( actionCollection()->action( "stop_job" )->isEnabled() );
     bool selected = (update->currentItem() != 0);
     bool nojob = !hasRunningJob && selected;
@@ -942,6 +939,7 @@ void CervisiaPart::slotCommit()
 
         if( protocol->startJob() )
         {
+            m_jobType = Commit;
             showJobStart(cmdline);
             connect( protocol, SIGNAL(jobFinished(bool, int)), update, SLOT(finishJob(bool, int)) );
             connect( protocol, SIGNAL(jobFinished(bool, int)), this, SLOT(slotJobFinished()) );
@@ -968,6 +966,24 @@ void CervisiaPart::slotRemove()
 }
 
 
+void CervisiaPart::slotFileProperties()
+{
+    QString filename;
+    update->getSingleSelection(&filename);
+    if( filename.isEmpty() )
+        return;
+
+    // Create URL from selected filename
+    QDir dir(sandbox);
+
+    KURL u;
+    u.setPath(dir.absFilePath(filename));
+
+    // show file properties dialog
+    (void)new KPropertiesDialog(u);
+}
+
+
 void CervisiaPart::updateSandbox(const QString &extraopt)
 {
     QStringList list = update->multipleSelection();
@@ -1354,7 +1370,7 @@ void CervisiaPart::slotCheckout()
 
     DCOPRef cvsJob = cvsService->checkout(dlg.workingDirectory(), dlg.repository(),
                                           dlg.module(), dlg.branch(), opt_pruneDirs,
-                                          dlg.alias(), dlg.exportOnly());
+                                          dlg.alias(), dlg.exportOnly(), dlg.recursive());
 
     // get command line from cvs job
     QString cmdline = cvsJob.call("cvsCommand()");
@@ -1606,6 +1622,14 @@ void CervisiaPart::slotJobFinished()
 
     disconnect( protocol, SIGNAL(receivedLine(QString)),
                 update,   SLOT(processUpdateLine(QString)) );
+
+    if( m_jobType == Commit )
+    {
+        KNotifyClient::event(widget()->parentWidget()->winId(), "cvs_commit_done",
+                             i18n("A CVS commit to repository %1 is done")
+                             .arg(repository));
+        m_jobType = Unknown;
+    }
 }
 
 
@@ -1710,110 +1734,10 @@ void CervisiaPart::setFilter()
                 str += "R";
         }
 
-#if KDE_IS_VERSION(3,1,90)
     filterLabel->setText(str);
-#endif
 }
 
 
-#if 0
-void CervisiaPart::parseStatus(QString pathname, QStrList list)
-{
-    char buf[512];
-    QString command;
-    QString dirpath;
-    QString name, statusstr, version;
-    enum { Begin, File, FileSep, WorkRev, FinalSep } state;
-
-    QString line = joinLine(list);
-
-    command = "cd ";
-    command += KProcess::quote(pathname);
-    command += " && " + cvsClient(repository) + " status -l ";
-    command += line;
-    //command += " 2>&1";
-
-    FILE *f = popen(QFile::encodeName(command), "r");
-    if (!f)
-        return;
-
-    state = Begin;
-    while (fgets(buf, sizeof buf, f))
-        {
-            QCString line = buf;
-            chomp(&line);
-            //DEBUGOUT( "Line: " << line );
-            switch (state)
-                {
-                case Begin:
-                    if (line.left(22) == "cvs status: Examining ")
-                        dirpath = line.right(line.length()-22);
-                    state = File;
-                    //DEBUGOUT( "state = file" );
-                    break;
-                case File:
-                    if (line.length() > 32 &&
-                        line.left(6) == "File: " &&
-                        line.mid(24, 8) == "Status: ")
-                    {
-                        name = line.mid(6, 18).stripWhiteSpace();
-                        if (dirpath != ".")
-                            name.prepend("/").prepend(dirpath);
-                        statusstr = line.right(line.length()-32)
-                                        .stripWhiteSpace();
-                        state = FileSep;
-                        //DEBUGOUT( "state = FileSep" );
-                    }
-                    break;
-                case FileSep:
-                    if (!line.isEmpty()) // Error
-                        state = WorkRev;
-                    //DEBUGOUT( "state = WorkRev" );
-                    break;
-                case WorkRev:
-                    if (line.left(21) == "   Working revision:\t")
-                    {
-                        int pos;
-                        version = line.right(line.length()-21);
-                        if ( (pos = version.find(" ")) != -1 )
-                            version.truncate(pos);
-                        state = FinalSep;
-                        //DEBUGOUT( "state = FinalSep" );
-                    }
-                    break;
-                case FinalSep:
-                    if (line == "")
-                    {
-                        //DEBUGOUT( "Adding: " << name <<
-                        //          "Status: " << statusstr << "Version: " << version );
-                        UpdateView::Status status = UpdateView::Unknown;
-                        if (statusstr == "Up-to-date")
-                            status = UpdateView::UpToDate;
-                        else if (statusstr == "Locally Modified")
-                            status = UpdateView::LocallyModified;
-                        else if (statusstr == "Locally Added")
-                            status = UpdateView::LocallyAdded;
-                        else if (statusstr == "Locally Removed")
-                            status = UpdateView::LocallyRemoved;
-                        else if (statusstr == "Needs Checkout")
-                            status = UpdateView::NeedsUpdate;
-                        else if (statusstr == "Needs Patch")
-                            status = UpdateView::NeedsPatch;
-                        else if (statusstr == "Needs Merge")
-                            status = UpdateView::NeedsMerge;
-                        else if (statusstr == "File had conflicts on merge")
-                            status = UpdateView::Conflict;
-                        //update->addEntry(status, name /*, version*/);
-                        state = Begin;
-                        //DEBUGOUT( "state = Begin" );
-                    }
-                }
-        }
-    pclose(f);
-}
-#endif
-
-
 void CervisiaPart::readSettings()
 {
     KConfig* config = CervisiaFactory::instance()->config();
@@ -1930,6 +1854,18 @@ CervisiaBrowserExtension::~CervisiaBrowserExtension()
 
 }
 
+
+void CervisiaBrowserExtension::setPropertiesActionEnabled(bool enabled)
+{
+    emit enableAction("properties", enabled);
+}
+
+
+void CervisiaBrowserExtension::properties()
+{
+    static_cast<CervisiaPart*>(parent())->slotFileProperties();
+}
+
 // Local Variables:
 // c-basic-offset: 4
 // End:
