@@ -16,13 +16,10 @@
 #ifndef CERVISIAPART_H
 #define CERVISIAPART_H
 
-#include <kdeversion.h>
 #include <kparts/part.h>
 #include <kparts/browserextension.h>
 #include <kparts/genericfactory.h>
-#if KDE_IS_VERSION(3,1,90)
 #include <kparts/statusbarextension.h>
-#endif
 
 #include "addremovedlg.h"
 #include "commitdlg.h"
@@ -41,9 +38,9 @@ class KAboutData;
 class KListView;
 class KRecentFilesAction;
 class CvsService_stub;
+class CervisiaBrowserExtension;
 
 
-#if KDE_IS_VERSION(3,1,90)
 class CervisiaStatusBarExtension : public KParts::StatusBarExtension
 {
 public:
@@ -52,7 +49,6 @@ public:
     {
     }
 };
-#endif
 
 
 /**
@@ -98,6 +94,7 @@ public slots:
     void slotAddBinary();
 
     void slotRemove();
+    void slotFileProperties();
     void slotRevert();
     void slotBrowseLog();
     //    void slotBrowseMultiLog();
@@ -154,6 +151,8 @@ protected:
     virtual void guiActivateEvent(KParts::GUIActivateEvent* event);
 
 private:
+    enum JobType { Unknown, Commit };
+
     void setupActions();
 
     void readSettings();
@@ -185,13 +184,14 @@ private:
     //for the Open Recent directories
     KRecentFilesAction *recent;
 
-    CvsService_stub* cvsService;
-#if KDE_IS_VERSION(3,1,90)
+    CvsService_stub*            cvsService;
     CervisiaStatusBarExtension* statusBar;
+    CervisiaBrowserExtension*   m_browserExt;
     QLabel*                     filterLabel;
-#endif
-	int                     m_editWithId;
-	Cervisia::EditWithMenu* m_currentEditMenu;
+
+    int                         m_editWithId;
+    Cervisia::EditWithMenu*     m_currentEditMenu;
+    JobType                     m_jobType;
 };
 
 typedef KParts::GenericFactory<CervisiaPart> CervisiaFactory;
@@ -206,6 +206,11 @@ class CervisiaBrowserExtension : public KParts::BrowserExtension
 public:
     CervisiaBrowserExtension( CervisiaPart * );
     ~CervisiaBrowserExtension();
+
+    void setPropertiesActionEnabled(bool enabled);
+
+public slots:
+    void properties();
 };
 
 #endif // CERVISIAPART_H
