@@ -20,6 +20,17 @@
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
+  In addition, as a special exception, the copyright holders give
+  permission to link the code of this program with any edition of
+  the Qt library by Trolltech AS, Norway (or with modified versions
+  of Qt that use the same license as Qt), and distribute linked
+  combinations including the two.  You must obey the GNU General
+  Public License in all respects for all of the code used other than
+  Qt. If you modify this file, you may extend this exception to
+  your version of the file, but you are not obligated to do so.  If
+  you do not wish to do so, delete this exception statement from
+  your version.
+
 **************************************************************************** */
 
 #include <kdatatool.h>
@@ -78,6 +89,9 @@ void KBabelView::aboutDictionary(const QString id)
 
 void KBabelView::informDictionary()
 {
+    if( isSearching () )
+	stopSearch ();
+
     if( msgstrEdit->isModified() )
 	dictBox->setTextChanged(_catalog->msgid(_currentIndex)
 	    ,*(_catalog->msgstr(_currentIndex).at(msgstrEdit->currentForm()))
@@ -849,7 +863,7 @@ void KBabelView::plural2msgstr()
 	    _catalog->applyBeginCommand( _currentIndex, Msgstr ,this);
 
 	    i=0;
-	    for( QStringList::Iterator it=msgs.begin() ; it!=msgs.end() ; it++ )
+	    for( QStringList::Iterator it=msgs.begin() ; it!=msgs.end() ; ++it )
 	    {
 		if( i!= msgstrEdit->currentForm() )
 		{
@@ -996,3 +1010,15 @@ void KBabelView::insertChar( QChar ch )
     msgstrEdit->setFocus();
 }
 
+void KBabelView::wordCount()
+{
+    uint total, untranslated, fuzzy;
+    
+    _catalog->wordCount( total, fuzzy, untranslated );
+    
+    KMessageBox::information( this
+	, i18n("Total words: %1\n\n"
+"Words in untranslated messages: %2\n\n"
+"Words in fuzzy messages: %3").arg(total).arg(untranslated).arg(fuzzy)
+	, i18n("Word Count") );
+}
