@@ -33,6 +33,7 @@
 #include <qpainter.h>
 #include <qpopupmenu.h>
 #include <qstyle.h>
+#include <qprocess.h>
 
 #include <kdebug.h>
 #include <klocale.h>
@@ -63,6 +64,7 @@
 #define DEFAULT_MAXCALLING   -1
 #define DEFAULT_SHOWSKIPPED  false
 #define DEFAULT_EXPANDCYCLES false
+#define DEFAULT_CLUSTERGROUPS false
 #define DEFAULT_DETAILLEVEL  1
 #define DEFAULT_LAYOUT       GraphOptions::TopDown
 #define DEFAULT_ZOOMPOS      Auto
@@ -115,7 +117,7 @@ int GraphEdgeList::compareItems(Item item1, Item item2)
 GraphNode::GraphNode()
 {
     _f=0;
-    self = cum = 0;
+    self = incl = 0;
     _cn = 0;
 
     _visible = false;
@@ -411,17 +413,17 @@ GraphExporter::GraphExporter()
     _go = this;
     _tmpFile = 0;
     _item = 0;
-    reset(0, 0, 0, QString::null);
+    reset(0, 0, 0, TraceItem::NoCostType, QString::null);
 }
 
 
 GraphExporter::GraphExporter(TraceData* d, TraceFunction* f, TraceCostType* ct,
-                             QString filename)
+                             TraceItem::CostType gt, QString filename)
 {
     _go = this;
     _tmpFile = 0;
     _item = 0;
-    reset(d, f, ct, filename);
+    reset(d, f, ct, gt, filename);
 }
 
 
@@ -437,7 +439,7 @@ GraphExporter::~GraphExporter()
 
 
 void GraphExporter::reset(TraceData*, TraceItem* i, TraceCostType* ct,
-                          QString filename)
+                          TraceItem::CostType gt, QString filename)
 {
   _graphCreated = false;
   _nodeMap.clear();
@@ -461,6 +463,7 @@ void GraphExporter::reset(TraceData*, TraceItem* i, TraceCostType* ct,
 
   _item = i;
   _costType = ct;
+  _groupType = gt;
   if (!i) return;
 
   if (filename.isEmpty()) {
@@ -493,24 +496,24 @@ void GraphExporter::createGraph()
       (_item->type() == TraceItem::FunctionCycle)) {
     TraceFunction* f = (TraceFunction*) _item;
 
-    double cum = f->cumulative()->subCost(_costType);
-    _realFuncLimit = cum * _go->funcLimit();
-    _realCallLimit = cum * _go->callLimit();
+    double incl = f->inclusive()->subCost(_costType);
+    _realFuncLimit = incl * _go->funcLimit();
+    _realCallLimit = incl * _go->callLimit();
 
     buildGraph(f, 0, true, 1.0); // down to callings
 
     // set costs of function back to 0, as it will be added again
     GraphNode& n = _nodeMap[f];
-    n.self = n.cum = 0.0;
+    n.self = n.incl = 0.0;
 
     buildGraph(f, 0, false, 1.0); // up to callers
   }
   else {
     TraceCall* c = (TraceCall*) _item;
 
-    double cum = c->subCost(_costType);
-    _realFuncLimit = cum * _go->funcLimit();
-    _realCallLimit = cum * _go->callLimit();
+    double incl = c->subCost(_costType);
+    _realFuncLimit = incl * _go->funcLimit();
+    _realCallLimit = incl * _go->callLimit();
 
     // create edge
     TraceFunction *caller, *called;
@@ -524,9 +527,9 @@ void GraphExporter::createGraph()
     e.cost  = c->subCost(_costType);
     e.count = c->callCount();
 
-    SubCost s = called->cumulative()->subCost(_costType);
+    SubCost s = called->inclusive()->subCost(_costType);
     buildGraph(called, 0, true,  e.cost / s); // down to callings
-    s = caller->cumulative()->subCost(_costType);
+    s = caller->inclusive()->subCost(_costType);
     buildGraph(caller, 0, false, e.cost / s); // up to callers
   }
 }
@@ -580,28 +583,68 @@ void GraphExporter::writeDot()
     *stream << QString("  overlap=false;\n  splines=true;\n");
   }
 
+  // for clustering
+  QMap<TraceCostItem*,QPtrList<GraphNode> > nLists;
+
   GraphNodeMap::Iterator nit;
   for ( nit = _nodeMap.begin();
         nit != _nodeMap.end(); ++nit ) {
     GraphNode& n = *nit;
 
-    if (n.cum <= _realFuncLimit) continue;
+    if (n.incl <= _realFuncLimit) continue;
 
-    QString abr = n.function()->prettyName();
-    if ((int)abr.length() > Configuration::maxSymbolLength())
-      abr = abr.left(Configuration::maxSymbolLength()) + "...";
-
-    *stream << QString("  F%1 [").arg((int)n.function(), 0, 16);
-    if (_useBox) {
-      // make label 3 lines for CallGraphView
-      *stream << QString("shape=box,label=\"** %1 **\\n**\\n%2\"];\n")
-        .arg(abr)
-        .arg(SubCost(n.cum).pretty());
+    // for clustering: get cost item group of function
+    TraceCostItem* g;
+    TraceFunction* f = n.function();
+    switch(_groupType) {
+    case TraceItem::Object: g = f->object(); break;
+    case TraceItem::Class:  g = f->cls(); break;
+    case TraceItem::File:   g = f->file(); break;
+    case TraceItem::FunctionCycle: g = f->cycle();  break;
+    default: g = 0; break;
     }
-    else
-      *stream << QString("label=\"%1\\n%2\"];\n")
-        .arg(abr)
-        .arg(SubCost(n.cum).pretty());
+    nLists[g].append(&n);
+  }
+
+  QMap<TraceCostItem*,QPtrList<GraphNode> >::Iterator lit;
+  int cluster = 0;
+  for ( lit = nLists.begin();
+        lit != nLists.end(); ++lit, cluster++ ) {
+    QPtrList<GraphNode>& l = lit.data();
+    TraceCostItem* i = lit.key();
+
+    if (_go->clusterGroups() && i) {
+      QString iabr = i->prettyName();
+      if ((int)iabr.length() > Configuration::maxSymbolLength())
+	iabr = iabr.left(Configuration::maxSymbolLength()) + "...";
+
+      *stream << QString("subgraph \"cluster%1\" { label=\"%2\";\n")
+	.arg(cluster).arg(iabr);
+    }
+
+    GraphNode* np;
+    for(np = l.first(); np; np = l.next() ) {
+      TraceFunction* f = np->function();
+
+      QString abr = f->prettyName();
+      if ((int)abr.length() > Configuration::maxSymbolLength())
+	abr = abr.left(Configuration::maxSymbolLength()) + "...";
+      
+      *stream << QString("  F%1 [").arg((int)f, 0, 16);
+      if (_useBox) {
+	// make label 3 lines for CallGraphView
+	*stream << QString("shape=box,label=\"** %1 **\\n**\\n%2\"];\n")
+	  .arg(abr)
+	  .arg(SubCost(np->incl).pretty());
+      }
+      else
+	*stream << QString("label=\"%1\\n%2\"];\n")
+	  .arg(abr)
+	  .arg(SubCost(np->incl).pretty());
+    }
+
+    if (_go->clusterGroups() && i)
+      *stream << QString("}\n");
   }
 
   GraphEdgeMap::Iterator eit;
@@ -622,8 +665,8 @@ void GraphExporter::writeDot()
     e.setCallerNode(&from);
     e.setCallingNode(&to);
 
-    if ((from.cum <= _realFuncLimit) ||
-        (to.cum <= _realFuncLimit)) continue;
+    if ((from.incl <= _realFuncLimit) ||
+        (to.incl <= _realFuncLimit)) continue;
 
     // remove dumped edges from n.callers/n.callings
     from.callings.removeRef(&e);
@@ -653,7 +696,7 @@ void GraphExporter::writeDot()
       for ( nit = _nodeMap.begin();
 	    nit != _nodeMap.end(); ++nit ) {
 	  GraphNode& n = *nit;
-	  if (n.cum <= _realFuncLimit) continue;
+	  if (n.incl <= _realFuncLimit) continue;
 
 	  costSum = countSum = 0.0;
 	  for (e=n.callers.first();e;e=n.callers.next()) {
@@ -782,18 +825,18 @@ void GraphExporter::buildGraph(TraceFunction* f, int d,
 	    << ") [to " << (toCallings ? "Callings":"Callers") << "]" << endl;
 #endif
 
-  double oldCum = 0.0;
+  double oldIncl = 0.0;
   GraphNode& n = _nodeMap[f];
   if (n.function() == 0) {
     n.setFunction(f);
   }
   else
-    oldCum = n.cum;
+    oldIncl = n.incl;
 
-  double cum = f->cumulative()->subCost(_costType) * factor;
-  n.cum  += cum;
+  double incl = f->inclusive()->subCost(_costType) * factor;
+  n.incl  += incl;
   n.self += f->subCost(_costType) * factor;
-  if (0) qDebug("  Added Cum. %f, now %f", cum, n.cum);
+  if (0) qDebug("  Added Incl. %f, now %f", incl, n.incl);
 
   // A negative depth limit means "unlimited"
   int maxDepth = toCallings ? _go->maxCallingDepth() : _go->maxCallerDepth();
@@ -803,22 +846,22 @@ void GraphExporter::buildGraph(TraceFunction* f, int d,
   }
 
   // if we just reached the limit by summing, do a DFS
-  // from here with full cum. cost because of previous cutoffs
-  if ((n.cum >= _realFuncLimit) && (oldCum < _realFuncLimit)) cum = n.cum;
+  // from here with full incl. cost because of previous cutoffs
+  if ((n.incl >= _realFuncLimit) && (oldIncl < _realFuncLimit)) incl = n.incl;
 
   if (f->cycle()) {
     // for cycles members, we never stop on first visit, but always on 2nd
     // note: a 2nd visit never should happen, as we don't follow inner-cycle
     //       calls
-    if (oldCum > 0.0) {
+    if (oldIncl > 0.0) {
       if (0) qDebug("  Cutoff, 2nd visit to Cycle Member");
       // and takeback cost addition, as it's added twice
-      n.cum  = oldCum;
+      n.incl  = oldIncl;
       n.self -= f->subCost(_costType) * factor;
       return;
     }
   }
-  else if (cum <= _realFuncLimit) {
+  else if (incl <= _realFuncLimit) {
     if (0) qDebug("  Cutoff, below limit");
     return;
   }
@@ -887,7 +930,7 @@ void GraphExporter::buildGraph(TraceFunction* f, int d,
     }
 
     // if we just reached the call limit (=func limit by summing, do a DFS
-    // from here with full cum. cost because of previous cutoffs
+    // from here with full incl. cost because of previous cutoffs
     if ((e.cost >= _realCallLimit) && (oldCost < _realCallLimit)) cost = e.cost;
     if (cost < _realCallLimit) {
       if (0) qDebug("  Edge Cutoff, limit not reached");
@@ -896,9 +939,9 @@ void GraphExporter::buildGraph(TraceFunction* f, int d,
 
     SubCost s;
     if (call->inCycle())
-      s = f2->cycle()->cumulative()->subCost(_costType);
+      s = f2->cycle()->inclusive()->subCost(_costType);
     else
-      s = f2->cumulative()->subCost(_costType);
+      s = f2->inclusive()->subCost(_costType);
     SubCost v = call->subCost(_costType);
     buildGraph(f2, d+1, toCallings, factor * v / s);
   }
@@ -991,9 +1034,9 @@ CanvasNode::CanvasNode(CallGraphView* v, GraphNode* n,
     if (_view->topLevel()->showExpanded()) {
       if (_view->activeFunction()) {
         if (_view->activeFunction()->cycle())
-          totalCost = _view->activeFunction()->cycle()->cumulative();
+          totalCost = _view->activeFunction()->cycle()->inclusive();
         else
-          totalCost = _view->activeFunction()->cumulative();
+          totalCost = _view->activeFunction()->inclusive();
       }
       else
         totalCost = (TraceCost*) _view->activeItem();
@@ -1001,13 +1044,13 @@ CanvasNode::CanvasNode(CallGraphView* v, GraphNode* n,
     else
 	totalCost = _view->data();
     double total = totalCost->subCost(_view->costType());
-    double cumP  = 100.0 * n->cum / total;
+    double inclP  = 100.0 * n->incl / total;
     if (_view->topLevel()->showPercentage())
 	setText(1, QString("%1 %")
-		.arg(cumP, 0, 'f', Configuration::percentPrecision()));
+		.arg(inclP, 0, 'f', Configuration::percentPrecision()));
     else
-	setText(1, SubCost(n->cum).pretty());
-    setPixmap(1, percentagePixmap(25,10,(int)(cumP+.5), Qt::blue, true));
+	setText(1, SubCost(n->incl).pretty());
+    setPixmap(1, percentagePixmap(25,10,(int)(inclP+.5), Qt::blue, true));
 }
 
 void CanvasNode::setSelected(bool s)
@@ -1071,9 +1114,9 @@ CanvasEdgeLabel::CanvasEdgeLabel(CallGraphView* v, CanvasEdge* ce,
     if (_view->topLevel()->showExpanded()) {
       if (_view->activeFunction()) {
         if (_view->activeFunction()->cycle())
-          totalCost = _view->activeFunction()->cycle()->cumulative();
+          totalCost = _view->activeFunction()->cycle()->inclusive();
         else
-          totalCost = _view->activeFunction()->cumulative();
+          totalCost = _view->activeFunction()->inclusive();
       }
       else
         totalCost = (TraceCost*) _view->activeItem();
@@ -1081,13 +1124,13 @@ CanvasEdgeLabel::CanvasEdgeLabel(CallGraphView* v, CanvasEdge* ce,
     else
         totalCost = _view->data();
     double total = totalCost->subCost(_view->costType());
-    double cumP  = 100.0 * e->cost / total;
+    double inclP  = 100.0 * e->cost / total;
     if (_view->topLevel()->showPercentage())
         setText(0, QString("%1 %")
-		.arg(cumP, 0, 'f', Configuration::percentPrecision()));
+		.arg(inclP, 0, 'f', Configuration::percentPrecision()));
     else
         setText(0, SubCost(e->cost).pretty());
-    setPixmap(0, percentagePixmap(25,10,(int)(cumP+.5), Qt::blue, true));
+    setPixmap(0, percentagePixmap(25,10,(int)(inclP+.5), Qt::blue, true));
 
     if (e->call() && (e->call()->isRecursion() || e->call()->inCycle())) {
 	QString icon = "undo";
@@ -1344,6 +1387,11 @@ CallGraphView::CallGraphView(TraceItemView* parentView,
 
   // tooltips...
   _tip = new CallGraphTip(this);
+
+  _renderProcess = 0;
+  _prevSelectedNode = 0;
+  connect(&_renderTimer, SIGNAL(timeout()),
+          this, SLOT(showRenderWarning()));
 }
 
 CallGraphView::~CallGraphView()
@@ -1379,6 +1427,8 @@ QString CallGraphView::whatsThis() const
 
 void CallGraphView::updateSizes(QSize s)
 {
+  if (!_canvas) return;
+
     if (s == QSize(0,0)) s = size();
 
     // the part of the canvas that should be visible
@@ -1664,6 +1714,11 @@ void CallGraphView::doUpdate(int changeType)
   if (changeType == groupTypeChanged) {
     if (!_canvas) return;
 
+    if (_clusterGroups) { 
+      refresh(); 
+      return;
+    }
+
     QCanvasItemList l = _canvas->allItems();
     QCanvasItemList::iterator it;
     for (it = l.begin();it != l.end(); ++it)
@@ -1676,7 +1731,7 @@ void CallGraphView::doUpdate(int changeType)
 
   if (changeType & dataChanged) {
       // invalidate old selection and graph part
-      _exporter.reset(_data, _activeItem, _costType);
+    _exporter.reset(_data, _activeItem, _costType, _groupType);
       _selectedNode = 0;
       _selectedEdge = 0;
   }
@@ -1684,39 +1739,77 @@ void CallGraphView::doUpdate(int changeType)
   refresh();
 }
 
+void CallGraphView::clear()
+{
+  if (!_canvas) return;
 
+  delete _canvas;
+  _canvas = 0;
+  _completeView->setCanvas(0);
+  setCanvas(0);
+}
+
+void CallGraphView::showText(QString s)
+{
+  clear();
+  _renderTimer.stop();
+
+  _canvas = new QCanvas(QApplication::desktop()->width(),
+			QApplication::desktop()->height());
+
+  QCanvasText* t = new QCanvasText(s, _canvas);
+  t->move(5, 5);
+  t->show();
+  center(0,0);
+  setCanvas(_canvas);
+  _canvas->update();
+  _completeView->hide();
+}
+
+void CallGraphView::showRenderWarning()
+{
+  QString s;
+
+  if (_renderProcess)
+    s =i18n("Warning: a long lasting graph layouting is in progress.\n"
+	    "Reduce node/edge limits for speedup.\n");
+  else
+    s = i18n("Layouting stopped.\n");
+  
+  s.append(i18n("The call graph has %1 nodes and %2 edges.\n")
+	   .arg(_exporter.nodeCount())
+	   .arg(_exporter.edgeCount()));
+
+  showText(s);
+}
+
+void CallGraphView::stopRendering()
+{
+  if (!_renderProcess) return;
+
+  _renderProcess->kill();
+  delete _renderProcess;
+  _renderProcess = 0;
+  _unparsedOutput = QString::null;
+
+  _renderTimer.start(200, true);
+}
 
 void CallGraphView::refresh()
 {
-    // we want to keep a selected node item at the same global position
-    GraphNode* oldNodeSelection = _selectedNode;
-    QPoint selectionPos = QPoint(-1,-1);
-    if (_selectedNode) {
-	QPoint center = _selectedNode->canvasNode()->boundingRect().center();
-	selectionPos  = contentsToViewport(center);
-    }
+  // trigger start of background rendering
+  if (_renderProcess) stopRendering();
 
-
-  viewport()->setUpdatesEnabled(false);
-
-  if (_canvas) {
-    delete _canvas;
-    _canvas = 0;
+  // we want to keep a selected node item at the same global position
+  _prevSelectedNode = _selectedNode;
+  _prevSelectedPos = QPoint(-1,-1);
+  if (_selectedNode) {
+    QPoint center = _selectedNode->canvasNode()->boundingRect().center();
+    _prevSelectedPos  = contentsToViewport(center);
   }
-  _completeView->setCanvas(0);
-  setCanvas(0);
 
   if (!_data || !_activeItem) {
-    _canvas = new QCanvas(QApplication::desktop()->width(),
-			  QApplication::desktop()->height());
-    QString s = i18n("No item activated for which to draw the call graph.\n");
-    QCanvasText* t = new QCanvasText(s, _canvas);
-    t->move(0, 0);
-    t->show();
-    center(0,0);
-    setCanvas(_canvas);
-    _canvas->update();
-    viewport()->setUpdatesEnabled(true);
+    showText(i18n("No item activated for which to draw the call graph."));
     return;
   }
 
@@ -1727,7 +1820,7 @@ void CallGraphView::refresh()
   case TraceItem::Call:
     break;
   default:
-    viewport()->setUpdatesEnabled(true);
+    showText(i18n("No call graph can be drawn for the active item."));
     return;
   }
 
@@ -1735,38 +1828,68 @@ void CallGraphView::refresh()
 
   _selectedNode = 0;
   _selectedEdge = 0;
-  _exporter.reset(_data, _activeItem, _costType);
+  _exporter.reset(_data, _activeItem, _costType, _groupType);
   _exporter.writeDot();
 
-  // the dot file name can not be influenced by the user, so there's
-  // no security problem...
-  QString popencmd;
+  _renderProcess = new QProcess(this);
   if (_layout == GraphOptions::Circular)
-      popencmd = QString("twopi %1 -Tplain")
-	  .arg(_exporter.filename());
+    _renderProcess->addArgument( "twopi" );
   else
-      popencmd = QString("dot %1 -Tplain")
-	  .arg(_exporter.filename());
+    _renderProcess->addArgument( "dot" );
+  _renderProcess->addArgument(_exporter.filename());
+  _renderProcess->addArgument( "-Tplain" );
 
-  if (1) kdDebug() << "Running '" << popencmd << "'..." << endl;
+  connect( _renderProcess, SIGNAL(readyReadStdout()),
+	   this, SLOT(readDotOutput()) );
+  connect( _renderProcess, SIGNAL(processExited()),
+	   this, SLOT(dotExited()) );
+
+  if (1) kdDebug() << "Running '" 
+		   << _renderProcess->arguments().join(" ")
+		   << "'..." << endl;
+
+  if ( !_renderProcess->start() ) {
+    QString e = i18n("No call graph is available because the following\n"
+		     "command cannot be run:\n'%1'\n")
+      .arg(_renderProcess->arguments().join(" "));
+    e += i18n("Please check that 'dot' is installed (package GraphViz).");
+    showText(e);
+
+    delete _renderProcess;
+    _renderProcess = 0;
 
-  FILE* iFILE = popen(popencmd.ascii(), "r");
-  if (iFILE == 0) {
-    kdError() << "Can't run dot!" << endl;
     return;
   }
 
+  _unparsedOutput = QString::null;
+
+  // layouting of more than seconds is dubious
+  _renderTimer.start(1000, true);  
+}
+
+void CallGraphView::readDotOutput()
+{
+  _unparsedOutput.append( _renderProcess->readStdout() );
+}
+
+void CallGraphView::dotExited()
+{
   QString line, cmd;
   CanvasNode *rItem;
   QCanvasEllipse* eItem;
   CanvasEdge* sItem;
   CanvasEdgeLabel* lItem;
-  QTextStream* dotStream = new QTextStream(iFILE, IO_ReadOnly);
+  QTextStream* dotStream;
   double scale = 1.0, scaleX = 1.0, scaleY = 1.0;
   double dotWidth, dotHeight;
   GraphNode* activeNode = 0;
   GraphEdge* activeEdge = 0;
 
+  _renderTimer.stop();
+  viewport()->setUpdatesEnabled(false);
+  clear();
+  dotStream = new QTextStream(_unparsedOutput, IO_ReadOnly);
+
   int lineno = 0;
   while (1) {
     line = dotStream->readLine();
@@ -2041,7 +2164,6 @@ void CallGraphView::refresh()
 
   }
   delete dotStream;
-  pclose(iFILE);
 
   // for keyboard navigation
   // TODO: Edge sorting. Better keep left-to-right edge order from dot now
@@ -2049,8 +2171,8 @@ void CallGraphView::refresh()
 
   if (!_canvas) {
     _canvas = new QCanvas(size().width(),size().height());
-    QString s = i18n("Error running the graph layout tool 'dot'.\n"
-		     "Please check that it is installed (package GraphViz).");
+    QString s = i18n("Error running the graph layouting tool.\n");
+    s += i18n("Please check that 'dot' is installed (package GraphViz).");
     QCanvasText* t = new QCanvasText(s, _canvas);
     t->move(5, 5);
     t->show();
@@ -2097,10 +2219,10 @@ void CallGraphView::refresh()
       int x = int(sNode->x() + sNode->width()/2);
       int y = int(sNode->y() + sNode->height()/2);
 
-      if (oldNodeSelection) {
-	  if (rect().contains(selectionPos))
-	      setContentsPos(x-selectionPos.x(),
-                             y-selectionPos.y());
+      if (_prevSelectedNode) {
+	  if (rect().contains(_prevSelectedPos))
+	      setContentsPos(x-_prevSelectedPos.x(),
+                             y-_prevSelectedPos.y());
 	  else
 	      ensureVisible(x,y,
                             sNode->width()/2+50, sNode->height()/2+50);
@@ -2120,6 +2242,9 @@ void CallGraphView::refresh()
 
   _canvas->update();
   viewport()->setUpdatesEnabled(true);
+
+  delete _renderProcess;
+  _renderProcess = 0;
 }
 
 void CallGraphView::contentsMovingSlot(int x, int y)
@@ -2274,6 +2399,11 @@ void CallGraphView::contentsContextMenuEvent(QContextMenuEvent* e)
     }
   }
 
+  if (_renderProcess) {
+    popup.insertItem(i18n("Stop Layouting"), 999);
+    popup.insertSeparator();
+  }
+
   addGoMenu(&popup);
   popup.insertSeparator();
 
@@ -2381,6 +2511,8 @@ void CallGraphView::contentsContextMenuEvent(QContextMenuEvent* e)
   gpopup.setItemChecked(130,_showSkipped);
   gpopup.insertItem(i18n("Inner-cycle Calls"), 131);
   gpopup.setItemChecked(131,_expandCycles);
+  gpopup.insertItem(i18n("Cluster Groups"), 132);
+  gpopup.setItemChecked(132,_clusterGroups);
 
   QPopupMenu vpopup;
   vpopup.setCheckable(true);
@@ -2421,13 +2553,15 @@ void CallGraphView::contentsContextMenuEvent(QContextMenuEvent* e)
   case 94: activated(cycle); break;
   case 95: activated(c); break;
 
+  case 999: stopRendering(); break;
+
   case 201:
       {
 	  TraceFunction* f = activeFunction();
 	  if (!f) break;
 
 	  QString n = QString("callgraph");
-	  GraphExporter ge(data(), f, costType(),
+	  GraphExporter ge(data(), f, costType(), groupType(),
 			   QString("%1.dot").arg(n));
 	  ge.setGraphOptions(this);
 	  ge.writeDot();
@@ -2480,6 +2614,7 @@ void CallGraphView::contentsContextMenuEvent(QContextMenuEvent* e)
 
   case 130: _showSkipped = !_showSkipped; break;
   case 131: _expandCycles = !_expandCycles; break;
+  case 132: _clusterGroups = !_clusterGroups; break;
 
   case 140: _detailLevel = 0; break;
   case 141: _detailLevel = 1; break;
@@ -2542,6 +2677,8 @@ void CallGraphView::readViewConfig(KConfig* c,
     _callLimit       = g->readDoubleNumEntry("CallLimit", DEFAULT_CALLLIMIT);
     _showSkipped     = g->readBoolEntry("ShowSkipped", DEFAULT_SHOWSKIPPED);
     _expandCycles    = g->readBoolEntry("ExpandCycles", DEFAULT_EXPANDCYCLES);
+    _clusterGroups   = g->readBoolEntry("ClusterGroups",
+					DEFAULT_CLUSTERGROUPS);
     _detailLevel     = g->readNumEntry("DetailLevel", DEFAULT_DETAILLEVEL);
     _layout          = GraphOptions::layout(g->readEntry("Layout",
 					   layoutString(DEFAULT_LAYOUT)));
@@ -2562,6 +2699,8 @@ void CallGraphView::saveViewConfig(KConfig* c,
     writeConfigEntry(&g, "CallLimit", _callLimit, DEFAULT_CALLLIMIT);
     writeConfigEntry(&g, "ShowSkipped", _showSkipped, DEFAULT_SHOWSKIPPED);
     writeConfigEntry(&g, "ExpandCycles", _expandCycles, DEFAULT_EXPANDCYCLES);
+    writeConfigEntry(&g, "ClusterGroups", _clusterGroups,
+		     DEFAULT_CLUSTERGROUPS);
     writeConfigEntry(&g, "DetailLevel", _detailLevel, DEFAULT_DETAILLEVEL);
     writeConfigEntry(&g, "Layout",
 		     layoutString(_layout), layoutString(DEFAULT_LAYOUT).utf8().data());
