@@ -28,40 +28,29 @@
 #include "../umldoc.h"
 #include "../class.h"
 #include "../enum.h"
-#include "../enumliteral.h"
-#include "../umlenumliterallist.h"
+#include "../classifierlistitem.h"
+#include "../umlclassifierlistitemlist.h"
+#include "../umltemplatelist.h"
 #include "../package.h"
 #include "../association.h"
 #include "../attribute.h"
 #include "../operation.h"
+#include "../template.h"
 #include "../umlnamespace.h"
 
 const QString AdaWriter::defaultPackageSuffix = "_Holder";
 
 AdaWriter::AdaWriter(UMLDoc *parent, const char *name)
 		: SimpleCodeGenerator(parent, name) {
-	indentlevel = 0;
-	// FIXME: Eventually we should fabricate an Indenter class
-	// that can be used by all code generators.
-	// NOTE: this now exists under new code gen system. Dont do here! -b.t.
 }
 
 AdaWriter::~AdaWriter() {}
 
-QString AdaWriter::spc() {
-	QString s;
-	if (m_indentation == " ")
-		s.fill(' ', indentlevel * m_indentationAmount);
-	else
-		s.fill('\t', indentlevel * m_indentationAmount);
-	return s;
-}
-
 /**
  * returns "Ada"
  */
 QString AdaWriter::getLanguage() {
-        return "Ada";
+	return "Ada";
 }
 
 /**
@@ -72,13 +61,13 @@ QString AdaWriter::getLanguage() {
 bool AdaWriter::isType (QString & type)
 {
    if(type == "AdaWriter")
-        return true;
+	return true;
    return false;
 }
 
 
 bool AdaWriter::isOOClass(UMLClassifier *c) {
-	Uml::UMLObject_Type ot = c->getBaseType();
+	Uml::Object_Type ot = c->getBaseType();
 	if (ot == Uml::ot_Interface)
 		return true;
 	if (ot == Uml::ot_Enum)
@@ -87,7 +76,7 @@ bool AdaWriter::isOOClass(UMLClassifier *c) {
 		kdDebug() << "AdaWriter::isOOClass: unknown object type " << ot << endl;
 		return false;
 	}
-	QString stype = c->getStereotype();
+	QString stype = c->getStereotype(false);
 	if (stype == "CORBAConstant" || stype == "CORBATypedef" ||
 	    stype == "CORBAStruct" || stype == "CORBAUnion")
 		return false;
@@ -131,13 +120,13 @@ QString AdaWriter::qualifiedName(UMLClassifier *c, bool withType, bool byValue)
 
 void AdaWriter::computeAssocTypeAndRole
        (UMLAssociation *a, QString& typeName, QString& roleName) {
-	UMLClassifier* c = (UMLClassifier*) m_doc->findUMLObject(a->getRoleId(A));
+	UMLClassifier* c = (UMLClassifier*) m_doc->findObjectById(a->getRoleId(Uml::A));
 	typeName = cleanName(c->getName());
-	if (! a->getMulti(A).isEmpty())
+	if (! a->getMulti(Uml::A).isEmpty())
 		typeName.append("_Array_Access");
-	roleName = a->getRoleName(A);
+	roleName = a->getRoleName(Uml::A);
 	if (roleName.isEmpty()) {
-		if (a->getMulti(A).isEmpty()) {
+		if (a->getMulti(Uml::A).isEmpty()) {
 			roleName = "M_";
 			roleName.append(typeName);
 		} else {
@@ -189,91 +178,124 @@ void AdaWriter::writeClass(UMLClassifier *c) {
 	if (imports.count()) {
 		for (UMLClassifier *con = imports.first(); con; con = imports.next()) {
 			if (con->getBaseType() != Uml::ot_Datatype)
-				ada << "with " << qualifiedName(con) << "; " << m_newLineEndingChars;
+				ada << "with " << qualifiedName(con) << "; " << m_endl;
 		}
-		ada << m_newLineEndingChars;
+		ada << m_endl;
 	}
 
+	// Generate generic formals.
+	UMLTemplateList template_params = c->getTemplateList();
+	if (template_params.count()) {
+		ada << getIndent() << "generic" << m_endl;
+		m_indentLevel++;
+		for (UMLTemplate *t = template_params.first(); t; t = template_params.next()) {
+			QString formalName = t->getName();
+			QString typeName = t->getTypeName();
+			if (typeName == "class") {
+				ada << getIndent() << "type " << formalName << " is tagged private;"
+				    << m_endl;
+			} else {
+				// Check whether it's a data type.
+				UMLClassifier *typeObj = t->getType();
+				if (typeObj == NULL) {
+					kdError() << "AdaWriter::writeClass(template_param "
+						  << typeName << "): typeObj is NULL" << endl;
+					ada << getIndent() << "type " << formalName << " is new " << typeName
+					    << " with private;  -- CHECK: codegen error"
+					    << m_endl;
+				} else if (typeObj->getBaseType() == Uml::ot_Datatype) {
+					ada << getIndent() << formalName << " : " << typeName << ";"
+					    << m_endl;
+				} else {
+					ada << getIndent() << "type " << typeName << " is new "
+					    << formalName << " with private;" << m_endl;
+				}
+			}
+		}
+		m_indentLevel--;
+	}
+
+	// Here comes the package proper.
 	QString pkg = qualifiedName(c);
-	ada << spc() << "package " << pkg << " is" << m_newLineEndingChars << m_newLineEndingChars;
-	indentlevel++;
+	ada << getIndent() << "package " << pkg << " is" << m_endl << m_endl;
+	m_indentLevel++;
 	if (c->getBaseType() == Uml::ot_Enum) {
 		UMLEnum *ue = static_cast<UMLEnum*>(c);
-		UMLEnumLiteralList litList = ue->getFilteredEnumLiteralList();
+		UMLClassifierListItemList litList = ue->getFilteredList(Uml::ot_EnumLiteral);
 		uint i = 0;
-		ada << spc() << "type " << classname << " is (" << m_newLineEndingChars;
-		indentlevel++;
-		for (UMLEnumLiteral *lit = litList.first(); lit; lit = litList.next()) {
+		ada << getIndent() << "type " << classname << " is (" << m_endl;
+		m_indentLevel++;
+		for (UMLClassifierListItem *lit = litList.first(); lit; lit = litList.next()) {
 			QString enumLiteral = cleanName(lit->getName());
-			ada << spc() << enumLiteral;
+			ada << getIndent() << enumLiteral;
 			if (++i < litList.count())
-				ada << "," << m_newLineEndingChars;
+				ada << "," << m_endl;
 		}
-		indentlevel--;
-		ada << ");" << m_newLineEndingChars << m_newLineEndingChars;
-		indentlevel--;
-		ada << spc() << "end " << pkg << ";" << m_newLineEndingChars << m_newLineEndingChars;
+		m_indentLevel--;
+		ada << ");" << m_endl << m_endl;
+		m_indentLevel--;
+		ada << getIndent() << "end " << pkg << ";" << m_endl << m_endl;
 		return;
 	}
 	if (! isOOClass(c)) {
-		QString stype = c->getStereotype();
+		QString stype = c->getStereotype(false);
 		if (stype == "CORBAConstant") {
-			ada << spc() << "-- " << stype << " is Not Yet Implemented" << m_newLineEndingChars << m_newLineEndingChars;
+			ada << getIndent() << "-- " << stype << " is Not Yet Implemented" << m_endl << m_endl;
 		} else if (myClass && myClass->isEnumeration()) {
-			UMLAttributeList *atl = myClass->getFilteredAttributeList();
+			UMLAttributeList atl = myClass->getAttributeList();
 			UMLAttribute *at;
-			ada << spc() << "type " << classname << " is (" << m_newLineEndingChars;
-			indentlevel++;
+			ada << getIndent() << "type " << classname << " is (" << m_endl;
+			m_indentLevel++;
 			uint i = 0;
-			for (at = atl->first(); at; at = atl->next()) {
+			for (at = atl.first(); at; at = atl.next()) {
 				QString enumLiteral = cleanName(at->getName());
-				ada << spc() << enumLiteral;
-				if (++i < atl->count())
-					ada << "," << m_newLineEndingChars;
+				ada << getIndent() << enumLiteral;
+				if (++i < atl.count())
+					ada << "," << m_endl;
 			}
-			indentlevel--;
-			ada << ");" << m_newLineEndingChars << m_newLineEndingChars;
+			m_indentLevel--;
+			ada << ");" << m_endl << m_endl;
 		} else if(stype == "CORBAStruct") {
 			if(myClass) {
-				UMLAttributeList *atl = myClass->getFilteredAttributeList();
+				UMLAttributeList atl = myClass->getAttributeList();
 				UMLAttribute *at;
-				ada << spc() << "type " << classname << " is record" << m_newLineEndingChars;
-				indentlevel++;
-				for (at = atl->first(); at; at = atl->next()) {
+				ada << getIndent() << "type " << classname << " is record" << m_endl;
+				m_indentLevel++;
+				for (at = atl.first(); at; at = atl.next()) {
 					QString name = cleanName(at->getName());
 					QString typeName = at->getTypeName();
-					ada << spc() << name << " : " << typeName;
+					ada << getIndent() << name << " : " << typeName;
 					QString initialVal = at->getInitialValue();
 					if (initialVal.latin1() && ! initialVal.isEmpty())
 						ada << " := " << initialVal;
-					ada << ";" << m_newLineEndingChars;
+					ada << ";" << m_endl;
 				}
-				indentlevel--;
-				ada << spc() << "end record;" << m_newLineEndingChars << m_newLineEndingChars;
+				m_indentLevel--;
+				ada << getIndent() << "end record;" << m_endl << m_endl;
 			}
 		} else if(stype == "CORBAUnion") {
-			ada << spc() << "-- " << stype << " is Not Yet Implemented" << m_newLineEndingChars << m_newLineEndingChars;
+			ada << getIndent() << "-- " << stype << " is Not Yet Implemented" << m_endl << m_endl;
 		} else if(stype == "CORBATypedef") {
-			ada << spc() << "-- " << stype << " is Not Yet Implemented" << m_newLineEndingChars << m_newLineEndingChars;
+			ada << getIndent() << "-- " << stype << " is Not Yet Implemented" << m_endl << m_endl;
 		} else {
-			ada << spc() << "-- " << stype << ": Unknown stereotype" << m_newLineEndingChars << m_newLineEndingChars;
+			ada << getIndent() << "-- " << stype << ": Unknown stereotype" << m_endl << m_endl;
 		}
-		indentlevel--;
-		ada << spc() << "end " << pkg << ";" << m_newLineEndingChars << m_newLineEndingChars;
+		m_indentLevel--;
+		ada << getIndent() << "end " << pkg << ";" << m_endl << m_endl;
 		return;
 	}
 
 	// Write class Documentation if non-empty or if force option set.
 	if (forceDoc() || !c->getDoc().isEmpty()) {
-		ada << "--" << m_newLineEndingChars;
+		ada << "--" << m_endl;
 		ada << "-- class " << classname << endl;
 		ada << formatDoc(c->getDoc(), "-- ");
-		ada << m_newLineEndingChars;
+		ada << m_endl;
 	}
 
 	UMLClassifierList superclasses = c->getSuperClasses();
 
-	ada << spc() << "type Object is ";
+	ada << getIndent() << "type Object is ";
 	if (c->getAbstract())
 		ada << "abstract ";
 	if (superclasses.isEmpty()) {
@@ -283,40 +305,40 @@ void AdaWriter::writeClass(UMLClassifier *c) {
 		UMLClassifier* parent = superclasses.first();
 		ada << "new " << qualifiedName(parent) << ".Object with ";
 	}
-	ada << "private;" << m_newLineEndingChars << m_newLineEndingChars;
-	ada << spc() << "type Object_Ptr is access all Object'Class;" << m_newLineEndingChars << m_newLineEndingChars;
+	ada << "private;" << m_endl << m_endl;
+	ada << getIndent() << "type Object_Ptr is access all Object'Class;" << m_endl << m_endl;
 
 	// Generate accessors for public attributes.
-	UMLAttributeList *atl = NULL;
-        if(myClass) {
+	UMLAttributeList atl;
+	if(myClass) {
 		UMLAttributeList atpub;
 		atpub.setAutoDelete(false);
 
-		atl = myClass->getFilteredAttributeList();
+		atl = myClass->getAttributeList();
 
 		UMLAttribute *at;
-		for (at = atl->first(); at; at = atl->next()) {
+		for (at = atl.first(); at; at = atl.next()) {
 			if (at->getScope() == Uml::Public)
 				atpub.append(at);
 		}
 		if (forceSections() || atpub.count())
-			ada << spc() << "-- Accessors for public attributes:" << m_newLineEndingChars << m_newLineEndingChars;
+			ada << getIndent() << "-- Accessors for public attributes:" << m_endl << m_endl;
 		for (at = atpub.first(); at; at = atpub.next()) {
 			QString member = cleanName(at->getName());
-			ada << spc() << "procedure Set_" << member << " (";
+			ada << getIndent() << "procedure Set_" << member << " (";
 			if (! at->getStatic())
 				ada << "Self : access Object; ";
-			ada << "To : " << at->getTypeName() << ");" << m_newLineEndingChars;
-			ada << spc() << "function  Get_" << member;
+			ada << "To : " << at->getTypeName() << ");" << m_endl;
+			ada << getIndent() << "function  Get_" << member;
 			if (! at->getStatic())
 				ada << " (Self : access Object)";
-			ada << " return " << at->getTypeName() << ";" << m_newLineEndingChars
-<< m_newLineEndingChars;
+			ada << " return " << at->getTypeName() << ";" << m_endl
+<< m_endl;
 		}
 	}
 
 	// Generate public operations.
-	UMLOperationList opl(c->getFilteredOperationsList());
+	UMLOperationList opl(c->getOpList());
 	UMLOperationList oppub;
 	oppub.setAutoDelete(false);
 	UMLOperation *op;
@@ -325,47 +347,47 @@ void AdaWriter::writeClass(UMLClassifier *c) {
 			oppub.append(op);
 	}
 	if (forceSections() || oppub.count())
-		ada << spc() << "-- Public methods:" << m_newLineEndingChars << m_newLineEndingChars;
+		ada << getIndent() << "-- Public methods:" << m_endl << m_endl;
 	for (op = oppub.first(); op; op = oppub.next())
 		writeOperation(op, ada);
 
-	indentlevel--;
-	ada << spc() << "private" << m_newLineEndingChars << m_newLineEndingChars;
-	indentlevel++;
+	m_indentLevel--;
+	ada << getIndent() << "private" << m_endl << m_endl;
+	m_indentLevel++;
 
 	// Generate auxiliary declarations for multiplicity of associations
 	UMLAssociationList aggregations = c->getAggregations();
 	if (!aggregations.isEmpty()) {
 		for (UMLAssociation *a = aggregations.first(); a; a = aggregations.next()) {
-			if (a->getMulti(A).isEmpty())
+			if (a->getMulti(Uml::A).isEmpty())
 				continue;
-			UMLClassifier* other = (UMLClassifier*) m_doc->findUMLObject(a->getRoleId(A));
+			UMLClassifier* other = (UMLClassifier*) m_doc->findObjectById(a->getRoleId(Uml::A));
 			QString member = cleanName(other->getName());
 			// Handling of packages is missing here
 			// A test and error action is missing here for !isOOClass()
-			ada << spc() << "type " << member << "_Array is array"
-			    << " (Positive range <>) of " << member << ".Object_Ptr;" << m_newLineEndingChars;
-			ada << spc() << "type " << member << "_Array_Access is access "
-			    << member << "_array;" << m_newLineEndingChars << m_newLineEndingChars;
+			ada << getIndent() << "type " << member << "_Array is array"
+			    << " (Positive range <>) of " << member << ".Object_Ptr;" << m_endl;
+			ada << getIndent() << "type " << member << "_Array_Access is access "
+			    << member << "_array;" << m_endl << m_endl;
 		}
 	}
 	UMLAssociationList compositions = c->getCompositions();
 	if (!compositions.isEmpty()) {
 		for (UMLAssociation *a = compositions.first(); a; a = compositions.next()) {
-			if (a->getMulti(A).isEmpty())
+			if (a->getMulti(Uml::A).isEmpty())
 				continue;
-			UMLObject *other = m_doc->findUMLObject(a->getRoleId(A));
+			UMLObject *other = m_doc->findObjectById(a->getRoleId(Uml::A));
 			QString member = cleanName(other->getName());
 			// Handling of packages is missing here
 			// Treatment of !isOOClass() is missing here
-			ada << spc() << "type " << member << "_Array is array"
-			    << " (Positive range <>) of " << member << ".Object;" << m_newLineEndingChars;
-			ada << spc() << "type " << member << "_Array_Access is access "
-			    << member << "_array;" << m_newLineEndingChars << m_newLineEndingChars;
+			ada << getIndent() << "type " << member << "_Array is array"
+			    << " (Positive range <>) of " << member << ".Object;" << m_endl;
+			ada << getIndent() << "type " << member << "_Array_Access is access "
+			    << member << "_array;" << m_endl << m_endl;
 		}
 	}
 
-	ada << spc() << "type Object is ";
+	ada << getIndent() << "type Object is ";
 	if (c->getAbstract())
 		ada << "abstract ";
 	if (superclasses.isEmpty()) {
@@ -375,65 +397,65 @@ void AdaWriter::writeClass(UMLClassifier *c) {
 		UMLClassifier* parent = superclasses.first();
 		ada << "new " << qualifiedName(parent) << ".Object with ";
 	}
-	ada << "record" << m_newLineEndingChars;
-	indentlevel++;
+	ada << "record" << m_endl;
+	m_indentLevel++;
 
 	if (forceSections() || !aggregations.isEmpty()) {
-		ada << spc() << "-- Aggregations:" << m_newLineEndingChars;
+		ada << getIndent() << "-- Aggregations:" << m_endl;
 		for (UMLAssociation *a = aggregations.first(); a; a = aggregations.next()) {
 			QString typeName, roleName;
 			computeAssocTypeAndRole(a, typeName, roleName);
-			ada << spc() << roleName << " : " << typeName << ";" << m_newLineEndingChars;
+			ada << getIndent() << roleName << " : " << typeName << ";" << m_endl;
 		}
 		ada << endl;
 	}
 	if (forceSections() || !compositions.isEmpty()) {
-		ada << spc() << "-- Compositions:" << m_newLineEndingChars;
+		ada << getIndent() << "-- Compositions:" << m_endl;
 		for (UMLAssociation *a = compositions.first(); a; a = compositions.next()) {
 			QString typeName, roleName;
 			computeAssocTypeAndRole(a, typeName, roleName);
-			ada << spc() << roleName << " : " << typeName << ";" << m_newLineEndingChars;
+			ada << getIndent() << roleName << " : " << typeName << ";" << m_endl;
 		}
 		ada << endl;
 	}
 
-	if (myClass && (forceSections() || atl->count())) {
-		ada << spc() << "-- Attributes:" << m_newLineEndingChars;
+	if (myClass && (forceSections() || atl.count())) {
+		ada << getIndent() << "-- Attributes:" << m_endl;
 		UMLAttribute *at;
-		for (at = atl->first(); at; at = atl->next()) {
+		for (at = atl.first(); at; at = atl.next()) {
 			if (at->getStatic())
 				continue;
-			ada << spc() << cleanName(at->getName()) << " : "
+			ada << getIndent() << cleanName(at->getName()) << " : "
 			    << at->getTypeName();
 			if (at && at->getInitialValue().latin1() && ! at->getInitialValue().isEmpty())
 				ada << " := " << at->getInitialValue();
-			ada << ";" << m_newLineEndingChars;
+			ada << ";" << m_endl;
 		}
 	}
-	bool haveAttrs = (myClass && atl->count());
+	bool haveAttrs = (myClass && atl.count());
 	if (aggregations.isEmpty() && compositions.isEmpty() && !haveAttrs)
-		ada << spc() << "null;" << m_newLineEndingChars;
-	indentlevel--;
-	ada << spc() << "end record;" << m_newLineEndingChars << m_newLineEndingChars;
+		ada << getIndent() << "null;" << m_endl;
+	m_indentLevel--;
+	ada << getIndent() << "end record;" << m_endl << m_endl;
 	if (haveAttrs) {
 		bool seen_static_attr = false;
-		for (UMLAttribute *at = atl->first(); at; at = atl->next()) {
+		for (UMLAttribute *at = atl.first(); at; at = atl.next()) {
 			if (! at->getStatic())
 				continue;
 			if (! seen_static_attr) {
-				ada << spc() << "-- Static attributes:" << m_newLineEndingChars;
+				ada << getIndent() << "-- Static attributes:" << m_endl;
 				seen_static_attr = true;
 			}
-			ada << spc();
+			ada << getIndent();
 			if (at->getScope() == Uml::Private)
 				ada << "-- Private:  ";
 			ada << cleanName(at->getName()) << " : " << at->getTypeName();
 			if (at && at->getInitialValue().latin1() && ! at->getInitialValue().isEmpty())
 				ada << " := " << at->getInitialValue();
-			ada << ";" << m_newLineEndingChars;
+			ada << ";" << m_endl;
 		}
 		if (seen_static_attr)
-			ada << m_newLineEndingChars;
+			ada << m_endl;
 	}
 	// Generate protected operations.
 	UMLOperationList opprot;
@@ -443,7 +465,7 @@ void AdaWriter::writeClass(UMLClassifier *c) {
 			opprot.append(op);
 	}
 	if (forceSections() || opprot.count())
-		ada << spc() << "-- Protected methods:" << m_newLineEndingChars << m_newLineEndingChars;
+		ada << getIndent() << "-- Protected methods:" << m_endl << m_endl;
 	for (op = opprot.first(); op; op = opprot.next())
 		writeOperation(op, ada);
 
@@ -460,12 +482,12 @@ void AdaWriter::writeClass(UMLClassifier *c) {
 			oppriv.append(op);
 	}
 	if (forceSections() || oppriv.count())
-		ada << spc() << "-- Private methods:" << m_newLineEndingChars << m_newLineEndingChars;
+		ada << getIndent() << "-- Private methods:" << m_endl << m_endl;
 	for (op = oppriv.first(); op; op = oppriv.next())
 		writeOperation(op, ada, true);
 
-	indentlevel--;
-	ada << spc() << "end " << pkg << ";" << m_newLineEndingChars << m_newLineEndingChars;
+	m_indentLevel--;
+	ada << getIndent() << "end " << pkg << ";" << m_endl << m_endl;
 	file.close();
 	emit codeGenerated(c, true);
 }
@@ -473,10 +495,10 @@ void AdaWriter::writeClass(UMLClassifier *c) {
 
 void AdaWriter::writeOperation(UMLOperation *op, QTextStream &ada, bool is_comment) {
 	UMLAttributeList *atl = op->getParmList();
-	QString rettype = op->getReturnType();
+	QString rettype = op->getTypeName();
 	bool use_procedure = (rettype.isEmpty() || rettype == "void");
 
-	ada << spc();
+	ada << getIndent();
 	if (is_comment)
 		ada << "-- ";
 	if (use_procedure)
@@ -489,20 +511,20 @@ void AdaWriter::writeOperation(UMLOperation *op, QTextStream &ada, bool is_comme
 	if (! op->getStatic()) {
 		ada << "Self : access Object";
 		if (atl->count())
-			ada << ";" << m_newLineEndingChars;
+			ada << ";" << m_endl;
 	}
 	if (atl->count()) {
 		uint i = 0;
-		indentlevel++;
+		m_indentLevel++;
 		for (UMLAttribute *at = atl->first(); at; at = atl->next()) {
-			ada << spc();
+			ada << getIndent();
 			if (is_comment)
 				ada << "-- ";
 			ada << cleanName(at->getName()) << " : ";
-			Uml::Parameter_Kind pk = at->getParmKind();
-			if (pk == Uml::pk_Out)
+			Uml::Parameter_Direction pk = at->getParmKind();
+			if (pk == Uml::pd_Out)
 				ada << "out ";
-			else if (pk == Uml::pk_InOut)
+			else if (pk == Uml::pd_InOut)
 				ada << "in out ";
 			else
 				ada << "in ";
@@ -510,15 +532,15 @@ void AdaWriter::writeOperation(UMLOperation *op, QTextStream &ada, bool is_comme
 			if (! at->getInitialValue().isEmpty())
 				ada << " := " << at->getInitialValue();
 			if (++i < atl->count()) //FIXME gcc warning
-				ada << ";" << m_newLineEndingChars;
+				ada << ";" << m_endl;
 		}
-		indentlevel--;
+		m_indentLevel--;
 	}
 	if (! (op->getStatic() && atl->count() == 0))
 		ada << ")";
 	if (! use_procedure)
 		ada << " return " << rettype;
-	ada << " is abstract;" << m_newLineEndingChars << m_newLineEndingChars;
+	ada << " is abstract;" << m_endl << m_endl;
 	// TBH, we make the methods abstract here because we don't have the means
 	// for generating meaningful implementations.
 }
@@ -536,131 +558,6 @@ void AdaWriter::createDefaultDatatypes() {
 	m_doc->createDatatype("String");
 }
 
-/**
- * List of reserved keywords for this code generator.
- *
- * Just add new keywords, then mark all lines and
- * pipe it through the external 'sort' program.
- */
-static const char *ReservedWords[] = {
-  "abort",
-  "abs",
-  "abstract",
-  "accept",
-  "access",
-  "aliased",
-  "all",
-  "and",
-  "Argument_Error",
-  "array",
-  "Assert_Failure",
-  "at",
-  "begin",
-  "body",
-  "Boolean",
-  "case",
-  "Character",
-  "constant",
-  "Constraint_Error",
-  "Conversion_Error",
-  "Data_Error",
-  "declare",
-  "delay",
-  "delta",
-  "Dereference_Error",
-  "Device_Error",
-  "digits",
-  "do",
-  "Duration",
-  "else",
-  "elsif",
-  "end",
-  "End_Error",
-  "entry",
-  "exception",
-  "exit",
-  "false",
-  "Float",
-  "for",
-  "function",
-  "generic",
-  "goto",
-  "if",
-  "in",
-  "Index_Error",
-  "Integer",
-  "is",
-  "Layout_Error",
-  "Length_Error",
-  "limited",
-  "Long_Float",
-  "Long_Integer",
-  "Long_Long_Float",
-  "Long_Long_Integer",
-  "loop",
-  "mod",
-  "Mode_Error",
-  "Name_Error",
-  "Natural",
-  "new",
-  "not",
-  "null",
-  "of",
-  "or",
-  "others",
-  "out",
-  "package",
-  "Pattern_Error",
-  "Picture_Error",
-  "Pointer_Error",
-  "Positive",
-  "pragma",
-  "private",
-  "procedure",
-  "Program_Error",
-  "protected",
-  "raise",
-  "range",
-  "record",
-  "rem",
-  "renames",
-  "requeue",
-  "return",
-  "reverse",
-  "select",
-  "separate",
-  "Short_Float",
-  "Short_Integer",
-  "Short_Short_Float",
-  "Short_Short_Integer",
-  "Status_Error",
-  "Storage_Error",
-  "String",
-  "subtype",
-  "Tag_Error",
-  "tagged",
-  "task",
-  "Tasking_Error",
-  "terminate",
-  "Terminator_Error",
-  "then",
-  "Time_Error",
-  "Translation_Error",
-  "true",
-  "type",
-  "until",
-  "Update_Error",
-  "use",
-  "Use_Error",
-  "when",
-  "while",
-  "Wide_Character",
-  "Wide_String",
-  "with",
-  "xor",
-  NULL
-};
-
 /**
  * Check whether the given string is a reserved word for the
  * language of this code generator
@@ -668,22 +565,13 @@ static const char *ReservedWords[] = {
  * @param rPossiblyReservedKeyword  The string to check.
  */
 bool AdaWriter::isReservedKeyword(const QString & rPossiblyReservedKeyword) {
-  const char **tmpReservedWords = getReservedKeywords();
 
-  if (tmpReservedWords == NULL)
-  {
-    return false;
-  }
+	const QStringList keywords = reservedKeywords();
 
-  while (tmpReservedWords[0] != NULL) {
-		QString keyword(tmpReservedWords[0]);
-
-		if (keyword.lower() == rPossiblyReservedKeyword.lower()) {
+	QStringList::ConstIterator it;
+	for (it = keywords.begin(); it != keywords.end(); ++it)
+		if ((*it).lower() == rPossiblyReservedKeyword.lower())
 			return true;
-		}
-
-    tmpReservedWords++;
-	}
 
 	return false;
 }
@@ -691,10 +579,129 @@ bool AdaWriter::isReservedKeyword(const QString & rPossiblyReservedKeyword) {
 /**
  * get list of reserved keywords
  */
-const char **
-AdaWriter::getReservedKeywords() {
-  return ReservedWords;
+const QStringList AdaWriter::reservedKeywords() const {
+
+  static QStringList keywords;
+
+  if ( keywords.isEmpty() ) {
+    keywords.append( "abort" );
+    keywords.append( "abs" );
+    keywords.append( "abstract" );
+    keywords.append( "accept" );
+    keywords.append( "access" );
+    keywords.append( "aliased" );
+    keywords.append( "all" );
+    keywords.append( "and" );
+    keywords.append( "Argument_Error" );
+    keywords.append( "array" );
+    keywords.append( "Assert_Failure" );
+    keywords.append( "at" );
+    keywords.append( "begin" );
+    keywords.append( "body" );
+    keywords.append( "Boolean" );
+    keywords.append( "case" );
+    keywords.append( "Character" );
+    keywords.append( "constant" );
+    keywords.append( "Constraint_Error" );
+    keywords.append( "Conversion_Error" );
+    keywords.append( "Data_Error" );
+    keywords.append( "declare" );
+    keywords.append( "delay" );
+    keywords.append( "delta" );
+    keywords.append( "Dereference_Error" );
+    keywords.append( "Device_Error" );
+    keywords.append( "digits" );
+    keywords.append( "do" );
+    keywords.append( "Duration" );
+    keywords.append( "else" );
+    keywords.append( "elsif" );
+    keywords.append( "end" );
+    keywords.append( "End_Error" );
+    keywords.append( "entry" );
+    keywords.append( "exception" );
+    keywords.append( "exit" );
+    keywords.append( "false" );
+    keywords.append( "Float" );
+    keywords.append( "for" );
+    keywords.append( "function" );
+    keywords.append( "generic" );
+    keywords.append( "goto" );
+    keywords.append( "if" );
+    keywords.append( "in" );
+    keywords.append( "Index_Error" );
+    keywords.append( "Integer" );
+    keywords.append( "is" );
+    keywords.append( "Layout_Error" );
+    keywords.append( "Length_Error" );
+    keywords.append( "limited" );
+    keywords.append( "Long_Float" );
+    keywords.append( "Long_Integer" );
+    keywords.append( "Long_Long_Float" );
+    keywords.append( "Long_Long_Integer" );
+    keywords.append( "loop" );
+    keywords.append( "mod" );
+    keywords.append( "Mode_Error" );
+    keywords.append( "Name_Error" );
+    keywords.append( "Natural" );
+    keywords.append( "new" );
+    keywords.append( "not" );
+    keywords.append( "null" );
+    keywords.append( "of" );
+    keywords.append( "or" );
+    keywords.append( "others" );
+    keywords.append( "out" );
+    keywords.append( "package" );
+    keywords.append( "Pattern_Error" );
+    keywords.append( "Picture_Error" );
+    keywords.append( "Pointer_Error" );
+    keywords.append( "Positive" );
+    keywords.append( "pragma" );
+    keywords.append( "private" );
+    keywords.append( "procedure" );
+    keywords.append( "Program_Error" );
+    keywords.append( "protected" );
+    keywords.append( "raise" );
+    keywords.append( "range" );
+    keywords.append( "record" );
+    keywords.append( "rem" );
+    keywords.append( "renames" );
+    keywords.append( "requeue" );
+    keywords.append( "return" );
+    keywords.append( "reverse" );
+    keywords.append( "select" );
+    keywords.append( "separate" );
+    keywords.append( "Short_Float" );
+    keywords.append( "Short_Integer" );
+    keywords.append( "Short_Short_Float" );
+    keywords.append( "Short_Short_Integer" );
+    keywords.append( "Status_Error" );
+    keywords.append( "Storage_Error" );
+    keywords.append( "String" );
+    keywords.append( "subtype" );
+    keywords.append( "Tag_Error" );
+    keywords.append( "tagged" );
+    keywords.append( "task" );
+    keywords.append( "Tasking_Error" );
+    keywords.append( "terminate" );
+    keywords.append( "Terminator_Error" );
+    keywords.append( "then" );
+    keywords.append( "Time_Error" );
+    keywords.append( "Translation_Error" );
+    keywords.append( "true" );
+    keywords.append( "type" );
+    keywords.append( "until" );
+    keywords.append( "Update_Error" );
+    keywords.append( "use" );
+    keywords.append( "Use_Error" );
+    keywords.append( "when" );
+    keywords.append( "while" );
+    keywords.append( "Wide_Character" );
+    keywords.append( "Wide_String" );
+    keywords.append( "with" );
+    keywords.append( "xor" );
+  }
+
+  return keywords;
 }
 
 #include "adawriter.moc"
-
