@@ -1,3 +1,8 @@
+ /*
+  *  copyright (C) 2002-2004
+  *  Umbrello UML Modeller Authors <uml-devel@ uml.sf.net>
+  */
+
 /***************************************************************************
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -74,11 +79,11 @@ void LinePath::setAssociation(AssociationWidget * association ) {
 	cleanup();
 	m_pAssociation = association;
 	createHeadLines();
-	if( getAssocType() == at_Coll_Message )
+	if( getAssocType() == Uml::at_Coll_Message )
 		setupParallelLine();
 	UMLView * view =  (UMLView *)m_pAssociation -> parent();
-	connect( view, SIGNAL( sigLineColorChanged( int ) ), this, SLOT( slotLineColorChanged( int ) ) );
-	connect( view, SIGNAL( sigLineWidthChanged( int ) ), this, SLOT( slotLineWidthChanged( int ) ) );
+	connect( view, SIGNAL( sigLineColorChanged( Uml::IDType ) ), this, SLOT( slotLineColorChanged( Uml::IDType ) ) );
+	connect( view, SIGNAL( sigLineWidthChanged( Uml::IDType ) ), this, SLOT( slotLineWidthChanged( Uml::IDType ) ) );
 }
 
 QPoint LinePath::getPoint( int pointIndex ) {
@@ -94,7 +99,7 @@ QPoint LinePath::getPoint( int pointIndex ) {
 	return line -> startPoint();
 }
 
-bool LinePath::setPoint( int pointIndex, QPoint point ) {
+bool LinePath::setPoint( int pointIndex, const QPoint &point ) {
 	int count = m_LineList.count();
 	if( count == 0 || pointIndex > count  || pointIndex < 0)
 		return false;
@@ -126,7 +131,7 @@ bool LinePath::setPoint( int pointIndex, QPoint point ) {
 	return true;
 }
 
-bool LinePath::isPoint( int pointIndex, QPoint point, unsigned short delta) {
+bool LinePath::isPoint( int pointIndex, const QPoint &point, unsigned short delta) {
 	/* onLinePath doesn't return the index number we mean */
 	pointIndex--;
 	int count = m_LineList.count();
@@ -151,7 +156,7 @@ bool LinePath::isPoint( int pointIndex, QPoint point, unsigned short delta) {
 	return false;
 }
 
-bool LinePath::insertPoint( int pointIndex, QPoint point ) {
+bool LinePath::insertPoint( int pointIndex, const QPoint &point ) {
 	int count = m_LineList.count();
 	if( count == 0 )
 		return false;
@@ -198,7 +203,7 @@ bool LinePath::insertPoint( int pointIndex, QPoint point ) {
 	return true;
 }
 
-bool LinePath::removePoint( int pointIndex, QPoint point, unsigned short delta )
+bool LinePath::removePoint( int pointIndex, const QPoint &point, unsigned short delta )
 {
 	/* get the number of line segments */
 	int count = m_LineList.count();
@@ -258,7 +263,7 @@ bool LinePath::removePoint( int pointIndex, QPoint point, unsigned short delta )
 	return true;
 }
 
-bool LinePath::setStartEndPoints( QPoint start, QPoint end ) {
+bool LinePath::setStartEndPoints( const QPoint &start, const QPoint &end ) {
 	int count = m_LineList.count();
 
 	if( count == 0 ) {
@@ -280,12 +285,12 @@ int LinePath::count() {
 	return m_LineList.count() + 1;
 }
 
-int LinePath::onLinePath( QPoint position ) {
-	typedef QCanvasItemList ItemList;
-	ItemList list = getCanvas() -> collisions( position );
-	ItemList::iterator item_it;
+int LinePath::onLinePath( const QPoint &position ) {
+	QCanvasItemList list = getCanvas() -> collisions( position );
 	int index = 0;
-	for( item_it = list.begin(); item_it != list.end(); ++item_it ) {
+
+	QCanvasItemList::iterator end(list.end());
+	for(QCanvasItemList::iterator item_it(list.begin()); item_it != end; ++item_it ) {
 		if( ( index = m_LineList.findRef( (QCanvasLine*)*item_it ) ) != -1 )
 			return index + 1;
 	}//end for
@@ -310,31 +315,29 @@ void LinePath::setAssocType( Uml::Association_Type type ) {
 		delete m_pClearPoly;
 		m_pClearPoly = 0;
 	}
-	createHeadLines();
-	if( type == at_Coll_Message )
+	if( type == Uml::at_Coll_Message )
 		setupParallelLine();
+	else
+		createHeadLines();
 	update();
 }
 
 void LinePath::update() {
-	if( m_bHeadCreated ) {
+	if (getAssocType() == Uml::at_Coll_Message) {
+		if (m_bParallelLineCreated) {
+			calculateParallelLine();
+			updateParallelLine();
+		} else
+			setupParallelLine();
+	} else if (m_bHeadCreated) {
 		calculateHead();
 		updateHead();
 	} else {
 		createHeadLines();
 	}
-
-	if( getAssocType() == at_Coll_Message ) {
-		if( m_bParallelLineCreated ) {
-			calculateParallelLine();
-			updateParallelLine();
-		} else
-			setupParallelLine();
-
-	}
 }
 
-void LinePath::slotLineColorChanged( int viewID ) {
+void LinePath::slotLineColorChanged( Uml::IDType viewID ) {
 	if(m_pAssociation->getUMLView()->getID() != viewID) {
 		return;
 	}
@@ -342,7 +345,7 @@ void LinePath::slotLineColorChanged( int viewID ) {
 }
 
 
-void LinePath::setLineColor( QColor color ) {
+void LinePath::setLineColor( const QColor &color ) {
 	QCanvasLine * line = 0;
 	uint linewidth = 0;
 	LineListIt it( m_LineList );
@@ -364,9 +367,9 @@ void LinePath::setLineColor( QColor color ) {
 		++pit;
 	}
 
-	if( getAssocType() == at_Aggregation )
+	if( getAssocType() == Uml::at_Aggregation )
 		if (m_pClearPoly) m_pClearPoly -> setBrush( QBrush( white ) );
-	else if( getAssocType() == at_Composition )
+	else if( getAssocType() == Uml::at_Composition )
 		if (m_pClearPoly) m_pClearPoly -> setBrush( QBrush( color ) );
 
 	if( m_pCircle ) {
@@ -375,7 +378,7 @@ void LinePath::setLineColor( QColor color ) {
 	}
 }
 
-void LinePath::slotLineWidthChanged( int viewID ) {
+void LinePath::slotLineWidthChanged( Uml::IDType viewID ) {
 	if(m_pAssociation->getUMLView()->getID() != viewID) {
 		return;
 	}
@@ -459,30 +462,24 @@ void LinePath::setupSelected() {
 	rect -> setPen( QPen( blue ) );
 	rect -> setVisible( true );
 	m_RectList.append( rect );
+	update();
 }
 
 QPen LinePath::getPen() {
 	Uml::Association_Type type = getAssocType();
-	if( type == at_Dependency || type == at_Realization || type == at_Anchor )
+	if( type == Uml::at_Dependency || type == Uml::at_Realization || type == Uml::at_Anchor )
 		return QPen( getLineColor(), getLineWidth(), DashLine );
 	return QPen( getLineColor(), getLineWidth() );
 }
 
-#ifdef DEBUG_ASSOCLINES
-extern int calls_to_calc_head;
-#endif
-
 void LinePath::calculateHead() {
-#ifdef DEBUG_ASSOCLINES
-	calls_to_calc_head++;
-#endif
 	uint size = m_LineList.count();
 	QPoint farPoint;
 	int halfLength = 10;
 	double arrowAngle = 0.2618;   // 0.5 * atan(sqrt(3.0) / 3.0) = 0.2618
-	Association_Type at = getAssocType();
-	bool diamond = (at == at_Aggregation || at == at_Composition);
-	if (diamond || at == at_Containment) {
+	Uml::Association_Type at = getAssocType();
+	bool diamond = (at == Uml::at_Aggregation || at == Uml::at_Composition);
+	if (diamond || at == Uml::at_Containment) {
 		farPoint = getPoint(1);
 		m_EgdePoint = getPoint(0);
 		if (diamond) {
@@ -543,8 +540,8 @@ void LinePath::calculateHead() {
 
 	m_PointArray.setPoint(0, m_EgdePoint);
 	m_PointArray.setPoint(1, m_ArrowPointA);
-	if( getAssocType() == at_Realization ||
-	    getAssocType() == at_Generalization ) {
+	if( getAssocType() == Uml::at_Realization ||
+	    getAssocType() == Uml::at_Generalization ) {
 		m_PointArray.setPoint( 2, m_ArrowPointB );
 		m_PointArray.setPoint( 3, m_EgdePoint );
 	} else {
@@ -554,16 +551,18 @@ void LinePath::calculateHead() {
 		m_PointArray.setPoint(2, diamondFarPoint);
 		m_PointArray.setPoint(3, m_ArrowPointB);
 	}
+
 }
 
 void LinePath::updateHead() {
 	int count = m_HeadList.count();
 	QCanvasLine * line = 0;
+
 	switch( getAssocType() ) {
-		case at_State:
-		case at_Activity:
-		case at_UniAssociation:
-		case at_Dependency:
+		case Uml::at_State:
+		case Uml::at_Activity:
+		case Uml::at_UniAssociation:
+		case Uml::at_Dependency:
 			if( count < 2)
 				return;
 			line = m_HeadList.at( 0 );
@@ -573,8 +572,21 @@ void LinePath::updateHead() {
 			line -> setPoints( m_EgdePoint.x(), m_EgdePoint.y(), m_ArrowPointB.x(), m_ArrowPointB.y() );
 			break;
 
-		case at_Generalization:
-		case at_Realization:
+		case Uml::at_Relationship:
+			if (count < 2) {
+				return;
+			}
+			//FIXME fix for when association is on top/bottom of widget
+			line = m_HeadList.at( 0 );
+			line->setPoints( m_PointArray[2].x(), m_PointArray[2].y(),
+					 m_PointArray[0].x(), m_PointArray[0].y()-8 );
+
+			line = m_HeadList.at( 1 );
+			line->setPoints( m_PointArray[2].x(), m_PointArray[2].y(),
+					 m_PointArray[0].x(), m_PointArray[0].y()+8 );
+
+		case Uml::at_Generalization:
+		case Uml::at_Realization:
 			if( count < 3)
 				return;
 			line = m_HeadList.at( 0 );
@@ -588,8 +600,8 @@ void LinePath::updateHead() {
 			m_pClearPoly -> setPoints( m_PointArray );
 			break;
 
-		case at_Composition:
-		case at_Aggregation:
+		case Uml::at_Composition:
+		case Uml::at_Aggregation:
 			if( count < 4)
 				return;
 			line = m_HeadList.at( 0 );
@@ -606,7 +618,7 @@ void LinePath::updateHead() {
 			m_pClearPoly -> setPoints( m_PointArray );
 			break;
 
-		case at_Containment:
+		case Uml::at_Containment:
 			if (count < 1)
 				return;
 			line = m_HeadList.at( 0 );
@@ -620,14 +632,14 @@ void LinePath::updateHead() {
 	}
 }
 
-void LinePath::growHeadList(int by) {
+void LinePath::growList(LineList &list, int by) {
 	QPen pen( getLineColor(), getLineWidth() );
 	for (int i = 0; i < by; i++) {
 		QCanvasLine * line = new QCanvasLine( getCanvas() );
 		line -> setZ( 0 );
 		line -> setPen( pen );
 		line -> setVisible( true );
-		m_HeadList.append( line );
+		list.append( line );
 	}
 }
 
@@ -635,36 +647,37 @@ void LinePath::createHeadLines() {
 	m_HeadList.clear();
 	QCanvas * canvas = getCanvas();
 	switch( getAssocType() ) {
-		case at_Activity:
-		case at_State:
-		case at_Dependency:
-		case at_UniAssociation:
-			growHeadList( 2 );
+		case Uml::at_Activity:
+		case Uml::at_State:
+		case Uml::at_Dependency:
+		case Uml::at_UniAssociation:
+		case Uml::at_Relationship:
+			growList(m_HeadList, 2);
 			break;
 
-		case at_Generalization:
-		case at_Realization:
-			growHeadList( 3 );
+		case Uml::at_Generalization:
+		case Uml::at_Realization:
+			growList(m_HeadList, 3);
 			m_pClearPoly = new QCanvasPolygon( canvas );
 			m_pClearPoly -> setVisible( true );
 			m_pClearPoly -> setBrush( QBrush( white ) );
 			m_pClearPoly -> setZ( -1 );
 			break;
 
-		case at_Composition:
-		case at_Aggregation:
-			growHeadList( 4 );
+		case Uml::at_Composition:
+		case Uml::at_Aggregation:
+			growList(m_HeadList, 4);
 			m_pClearPoly = new QCanvasPolygon( canvas );
 			m_pClearPoly -> setVisible( true );
-			if( getAssocType() == at_Aggregation )
+			if( getAssocType() == Uml::at_Aggregation )
 				m_pClearPoly -> setBrush( QBrush( white ) );
 			else
 				m_pClearPoly -> setBrush( QBrush( getLineColor() ) );
 			m_pClearPoly -> setZ( -1 );
 			break;
 
-		case at_Containment:
-			growHeadList( 1 );
+		case Uml::at_Containment:
+			growList(m_HeadList, 1);
 			if (!m_pCircle) {
 				m_pCircle = new Circle( canvas, 6 );
 				m_pCircle->show();
@@ -727,25 +740,7 @@ void LinePath::calculateParallelLine() {
 
 void LinePath::setupParallelLine() {
 	m_ParallelList.clear();
-	QCanvas * canvas = getCanvas();
-	QPen pen( getLineColor(), getLineWidth() );
-	QCanvasLine * line = new QCanvasLine( canvas );
-	line -> setZ( 0 );
-	line -> setPen( pen );
-	line -> setVisible( true );
-	m_ParallelList.append( line );
-
-	line = new QCanvasLine( canvas );
-	line -> setZ( 0 );
-	line -> setPen( pen );
-	line -> setVisible( true );
-	m_ParallelList.append( line );
-
-	line = new QCanvasLine( canvas );
-	line -> setZ( 0 );
-	line -> setPen( pen );
-	line -> setVisible( true );
-	m_ParallelList.append( line );
+	growList(m_ParallelList, 3);
 	m_bParallelLineCreated = true;
 }
 
@@ -802,7 +797,7 @@ LinePath & LinePath::operator=( LinePath & rhs ) {
 QCanvas * LinePath::getCanvas() {
 	if( !m_pAssociation )
 		return 0;
-	UMLView * view =  (UMLView *)m_pAssociation -> parent();
+	const UMLView * view =  m_pAssociation->getUMLView();
 	return view -> canvas();
 }
 
@@ -850,8 +845,8 @@ void LinePath::cleanup() {
 	if( m_pAssociation ) {
 		UMLView * view =  (UMLView *)m_pAssociation -> parent();
 		if(view) {
-			disconnect( view, SIGNAL( sigLineColorChanged( int ) ), this, SLOT( slotLineColorChanged( int ) ) );
-			disconnect( view, SIGNAL( sigLineWidthChanged( int ) ), this, SLOT( slotLineWidthChanged( int ) ) );
+			disconnect( view, SIGNAL( sigLineColorChanged( Uml::IDType ) ), this, SLOT( slotLineColorChanged( Uml::IDType ) ) );
+			disconnect( view, SIGNAL( sigLineWidthChanged( Uml::IDType ) ), this, SLOT( slotLineWidthChanged( Uml::IDType ) ) );
 		}
 		m_pAssociation = NULL;
 	}
