@@ -20,9 +20,9 @@
 #include "interfacewidget.h"
 #include "floatingtext.h"
 #include "uml.h"
+#include "model_utils.h"
+#include "widget_utils.h"
 #include "umlview.h"
-#include "notewidget.h"
-#include "boxwidget.h"
 #include "statewidget.h"
 #include "activitywidget.h"
 #include "objectwidget.h"
@@ -73,6 +73,10 @@ ListPopupMenu::ListPopupMenu(QWidget *parent, Uml::ListView_Type type)
 			mt = mt_Deployment_Folder;
 			break;
 
+		case Uml::lvt_EntityRelationship_Folder:
+			mt = mt_EntityRelationship_Folder;
+			break;
+
 		case Uml::lvt_UseCase_Diagram:
 			mt = mt_UseCase_Diagram;
 			break;
@@ -105,6 +109,10 @@ ListPopupMenu::ListPopupMenu(QWidget *parent, Uml::ListView_Type type)
 			mt = mt_Deployment_Diagram;
 			break;
 
+		case Uml::lvt_EntityRelationship_Diagram:
+			mt = mt_EntityRelationship_Diagram;
+			break;
+
 		case Uml::lvt_Actor:
 			mt = mt_Actor;
 			break;
@@ -152,6 +160,19 @@ ListPopupMenu::ListPopupMenu(QWidget *parent, Uml::ListView_Type type)
 		case Uml::lvt_Operation:
 			mt = mt_Operation;
 			break;
+
+		case Uml::lvt_Template:
+			mt = mt_Template;
+			break;
+
+		case Uml::lvt_Entity:
+			mt = mt_Entity;
+			break;
+
+		case Uml::lvt_EntityAttribute:
+			mt = mt_EntityAttribute;
+			break;
+
 		default:   ;
 			//break;
 	}
@@ -168,7 +189,7 @@ ListPopupMenu::ListPopupMenu(QWidget * parent, UMLWidget * object,
 	//make menu for logical view
 	if(!object)
 		return;
-	Uml::UMLWidget_Type type = object -> getBaseType();
+	Uml::Widget_Type type = object -> getBaseType();
 
 	if(multi) {
 		if (unique && (type == Uml::wt_Class || type == Uml::wt_Interface)) {
@@ -233,6 +254,17 @@ ListPopupMenu::ListPopupMenu(QWidget * parent, UMLWidget * object,
 			insertStdItem(mt_Properties);
 			break;
 
+		case Uml::wt_Entity:
+			m_pInsert = new KPopupMenu(this,"New");
+			m_pInsert->insertItem(SmallIcon("source"), i18n("Entity Attribute..."), mt_EntityAttribute);
+			insertFileNew();
+			setupColor(object->getUseFillColour());
+			insertStdItems(true, type);
+			insertStdItem(mt_Rename);
+			insertStdItem(mt_Change_Font);
+			insertStdItem(mt_Properties);
+			break;
+
 		case Uml::wt_Datatype:
 		case Uml::wt_Package:
 		case Uml::wt_Component:
@@ -281,8 +313,6 @@ ListPopupMenu::ListPopupMenu(QWidget * parent, UMLWidget * object,
 			insertItem(i18n("Change Text..."), mt_Rename);
 			insertStdItem(mt_Delete);
 			insertStdItem(mt_Change_Font);
-			//insertItem(i18n("Link Documentation"), mt_Link_Docs);
-			//setItemChecked(mt_Link_Docs, ((NoteWidget*)object)->getLinkState());
 			break;
 
 		case Uml::wt_Box:
@@ -320,13 +350,22 @@ ListPopupMenu::ListPopupMenu(QWidget * parent, UMLWidget * object,
 		case Uml::wt_Text:
 			switch( (static_cast<FloatingText*>(object))->getRole() ) {
 				case Uml::tr_MultiB:
+					insertAssocItem(i18n("Change Multiplicity..."), mt_Rename_MultiB);
+					break;
 				case Uml::tr_MultiA:
+					insertAssocItem(i18n("Change Multiplicity..."), mt_Rename_MultiA);
+					break;
 				case Uml::tr_Name:
+					insertAssocItem(i18n("Change Name"), mt_Rename_Name);
+					break;
 				case Uml::tr_RoleAName:
+					insertAssocItem(i18n("Change Role A Name..."), mt_Rename_RoleAName);
+					break;
 				case Uml::tr_RoleBName:
+					insertAssocItem(i18n("Change Role B Name..."), mt_Rename_RoleBName);
+					break;
 				case Uml::tr_ChangeA:
 				case Uml::tr_ChangeB:
-					insertStdItem(mt_Delete_Association);
 					insertStdItem(mt_Change_Font);
 					insertStdItem(mt_Reset_Label_Positions);
 					insertStdItem(mt_Properties);
@@ -336,11 +375,6 @@ ListPopupMenu::ListPopupMenu(QWidget * parent, UMLWidget * object,
 				case Uml::tr_Coll_Message:
 				case Uml::tr_Seq_Message_Self:
 				case Uml::tr_Seq_Message:
-					// insertStdItem(mt_Cut);
-					// insertStdItem(mt_Copy);
-					// insertStdItem(mt_Paste);
-					// insertSeparator();
-					insertItem(SmallIcon( "editdelete"), i18n("Delete"), mt_Delete_Message);
 					insertStdItem(mt_Change_Font);
 					insertItem(SmallIcon( "filenew"), i18n("New Operation..."), mt_Operation);
 					insertItem(i18n("Select Operation..."), mt_Select_Operation);
@@ -437,12 +471,22 @@ void ListPopupMenu::insertStdItem(Menu_Type m)
 		m_pInsert->insertItem(m_pixmap[pm_Node], i18n("Node"), mt_Node);
 		break;
 	case mt_Deployment_Diagram:
-		m_pInsert->insertItem(BarIconSet("umbrello_diagram_deployment"),i18n("Deployment Diagram..."),
+		m_pInsert->insertItem(Umbrello::iconSet(Uml::dt_Deployment), i18n("Deployment Diagram..."),
 				      mt_Deployment_Diagram);
 		break;
+	case mt_Entity:
+		m_pInsert->insertItem(m_pixmap[pm_Entity], i18n("Entity"), mt_Entity);
+		break;
+	case mt_EntityRelationship_Diagram:
+		m_pInsert->insertItem(Umbrello::iconSet(Uml::dt_EntityRelationship), i18n("Entity Relationship Diagram..."),
+				      mt_EntityRelationship_Diagram);
+		break;
 	case mt_UseCase_Folder:
 		m_pInsert->insertItem(BarIcon("folder_new"), i18n("Folder"), mt_UseCase_Folder);
 		break;
+	case mt_EntityRelationship_Folder:
+		m_pInsert->insertItem(BarIcon("folder_new"), i18n("Folder"), mt_EntityRelationship_Folder);
+		break;
 	case mt_Actor:
 		m_pInsert->insertItem(m_pixmap[pm_Actor], i18n("Actor"), mt_Actor);
 		break;
@@ -450,15 +494,12 @@ void ListPopupMenu::insertStdItem(Menu_Type m)
 		m_pInsert->insertItem(m_pixmap[pm_Usecase], i18n("Use Case"), mt_UseCase);
 		break;
 	case mt_UseCase_Diagram:
-		m_pInsert->insertItem(BarIconSet("umbrello_diagram_usecase"), i18n("Use Case Diagram..."),
+		m_pInsert->insertItem(Umbrello::iconSet(Uml::dt_UseCase), i18n("Use Case Diagram..."),
 				      mt_UseCase_Diagram);
 		break;
 	case mt_FloatText:
 		m_pInsert->insertItem(m_pixmap[pm_Text], i18n("Text Line..." ), mt_FloatText);
 		break;
-	case mt_Delete_Association:
-		insertItem(SmallIcon("editdelete"), i18n("Delete Association"), mt_Delete_Association);
-		break;
 	case mt_Reset_Label_Positions:
 		insertItem(i18n("Reset Label Positions"), mt_Reset_Label_Positions);
 		break;
@@ -477,6 +518,9 @@ void ListPopupMenu::insertStdItem(Menu_Type m)
 	case mt_New_EnumLiteral:
 		insertItem(SmallIcon("source"), i18n("New Literal..."), mt_New_EnumLiteral);
 		break;
+	case mt_New_EntityAttribute:
+		insertItem(SmallIcon("source"), i18n("New Entity Attribute..."), mt_New_EntityAttribute);
+		break;
 	case mt_New_Activity:
 		m_pInsert->insertItem(SmallIcon("source"), i18n("Activity..."), mt_New_Activity);
 		break;
@@ -490,7 +534,7 @@ void ListPopupMenu::insertStdItem(Menu_Type m)
 }
 
 void ListPopupMenu::insertStdItems(bool insertLeadingSeparator /* = true */,
-				   Uml::UMLWidget_Type type /* = wt_UMLWidget */)
+				   Uml::Widget_Type type /* = wt_UMLWidget */)
 {
 	if (insertLeadingSeparator)
 		insertSeparator();
@@ -498,9 +542,9 @@ void ListPopupMenu::insertStdItems(bool insertLeadingSeparator /* = true */,
 	insertStdItem(mt_Copy);
 	insertStdItem(mt_Paste);
 	insertSeparator();
-	if (type == wt_UMLWidget)
+	if (type == Uml::wt_UMLWidget)
 		insertStdItem(mt_Rename);
-	else if (isCloneable(type))
+	else if (Umbrello::isCloneable(type))
 		insertStdItem(mt_Clone);
 	insertStdItem(mt_Delete);
 }
@@ -514,37 +558,25 @@ void ListPopupMenu::insertContainerItems(bool folderAndDiagrams) {
 	m_pInsert -> insertItem(m_pixmap[pm_Enum], i18n("Enum"), mt_Enum);
 	m_pInsert -> insertItem(m_pixmap[pm_Package], i18n("Package"), mt_Package);
 	if (folderAndDiagrams) {
-		m_pInsert->insertItem(BarIconSet("umbrello_diagram_class"), i18n("Class Diagram..."), mt_Class_Diagram);
-		m_pInsert->insertItem(BarIconSet("umbrello_diagram_state"), i18n("State Diagram..."), mt_State_Diagram);
-		m_pInsert->insertItem(BarIconSet("umbrello_diagram_activity"), i18n("Activity Diagram..."), mt_Activity_Diagram);
-		m_pInsert->insertItem(BarIconSet("umbrello_diagram_sequence"), i18n("Sequence Diagram..."), mt_Sequence_Diagram);
-		m_pInsert->insertItem(BarIconSet("umbrello_diagram_collaboration"), i18n("Collaboration Diagram..."), mt_Collaboration_Diagram);
+		m_pInsert->insertItem(Umbrello::iconSet(Uml::dt_Class), i18n("Class Diagram..."), mt_Class_Diagram);
+		m_pInsert->insertItem(Umbrello::iconSet(Uml::dt_State), i18n("State Diagram..."), mt_State_Diagram);
+		m_pInsert->insertItem(Umbrello::iconSet(Uml::dt_Activity), i18n("Activity Diagram..."), mt_Activity_Diagram);
+		m_pInsert->insertItem(Umbrello::iconSet(Uml::dt_Sequence), i18n("Sequence Diagram..."), mt_Sequence_Diagram);
+		m_pInsert->insertItem(Umbrello::iconSet(Uml::dt_Collaboration), i18n("Collaboration Diagram..."), mt_Collaboration_Diagram);
 	}
 	insertFileNew();
 }
 
-bool ListPopupMenu::isCloneable(Uml::UMLWidget_Type type)
-{
-	switch (type) {
-		case Uml::wt_Actor:
-		case Uml::wt_UseCase:
-		case Uml::wt_Class:
-		case Uml::wt_Interface:
-		case Uml::wt_Enum:
-		case Uml::wt_Datatype:
-		case Uml::wt_Package:
-		case Uml::wt_Component:
-		case Uml::wt_Node:
-		case Uml::wt_Artifact:
-			return true;
-		default:
-			return false;
-	}
+void ListPopupMenu::insertAssocItem(const QString &label, Menu_Type mt) {
+	insertItem(label, mt);
+	insertStdItem(mt_Change_Font);
+	insertStdItem(mt_Reset_Label_Positions);
+	insertStdItem(mt_Properties);
 }
 
 void ListPopupMenu::makeMultiClassifierPopup(ClassifierWidget *c)
 {
-	Uml::UMLWidget_Type type = c->getBaseType();
+	Uml::Widget_Type type = c->getBaseType();
 	ClassWidget *cls = NULL;
 
 	m_pShow = new KPopupMenu(this, "Show");
@@ -585,7 +617,7 @@ void ListPopupMenu::makeMultiClassifierPopup(ClassifierWidget *c)
 
 void ListPopupMenu::makeClassifierPopup(ClassifierWidget *c)
 {
-	Uml::UMLWidget_Type type = c->getBaseType();
+	Uml::Widget_Type type = c->getBaseType();
 	m_pInsert = new KPopupMenu(this,"New");
 	if (type == Uml::wt_Class)
 		m_pInsert->insertItem(SmallIcon( "CVpublic_var" ), i18n("Attribute..."), mt_Attribute);
@@ -653,14 +685,17 @@ Uml::Diagram_Type ListPopupMenu::convert_MT_DT(Menu_Type mt) {
 		case mt_Activity_Diagram:
 			type = Uml::dt_Activity;
 			break;
+		case mt_EntityRelationship_Diagram:
+			type = Uml::dt_EntityRelationship;
+			break;
 		default:
 			break;
 	}
 	return type;
 }
 
-Uml::UMLObject_Type ListPopupMenu::convert_MT_OT(Menu_Type mt) {
-	Uml::UMLObject_Type type =  Uml::ot_UMLObject;
+Uml::Object_Type ListPopupMenu::convert_MT_OT(Menu_Type mt) {
+	Uml::Object_Type type =  Uml::ot_UMLObject;
 
 	switch(mt) {
 		case mt_UseCase:
@@ -675,6 +710,9 @@ Uml::UMLObject_Type ListPopupMenu::convert_MT_OT(Menu_Type mt) {
 		case mt_Attribute:
 			type = Uml::ot_Attribute;
 			break;
+		case mt_EntityAttribute:
+			type = Uml::ot_EntityAttribute;
+			break;
 		case mt_Operation:
 			type = Uml::ot_Operation;
 			break;
@@ -708,6 +746,7 @@ void ListPopupMenu::setupMenu(Menu_Type type, UMLView* view) {
 	m_pixmap[pm_Object]      .load(dataDir+"object.png",        "PNG");
 	m_pixmap[pm_Component]   .load(dataDir+"component.png",     "PNG");
 	m_pixmap[pm_Node]        .load(dataDir+"node.png",          "PNG");
+	m_pixmap[pm_Entity]      .load(dataDir+"entity.png",        "PNG");
 	m_pixmap[pm_Artifact]    .load(dataDir+"artifact.png",      "PNG");
 	m_pixmap[pm_Text]        .load(dataDir+"text.png",          "PNG");
 
@@ -751,6 +790,17 @@ void ListPopupMenu::setupMenu(Menu_Type type, UMLView* view) {
 			insertStdItem(mt_Collapse_All);
 			break;
 
+		case mt_EntityRelationship_Model:
+			m_pInsert = new KPopupMenu(this, "New");
+			insertStdItem(mt_Entity);
+			insertFileNew();
+			insertSeparator();
+			insertStdItem(mt_Paste);
+			insertSeparator();
+			insertStdItem(mt_Expand_All);
+			insertStdItem(mt_Collapse_All);
+			break;
+
 		case mt_UseCase_View:
 			m_pInsert = new KPopupMenu(this,"New");
 			insertStdItem(mt_UseCase_Folder);
@@ -815,6 +865,18 @@ void ListPopupMenu::setupMenu(Menu_Type type, UMLView* view) {
 			insertStdItem(mt_Collapse_All);
 			break;
 
+		case mt_EntityRelationship_Folder:
+			m_pInsert = new KPopupMenu(this,"New");
+			insertStdItem(mt_EntityRelationship_Folder);
+			insertStdItem(mt_Entity);
+			insertStdItem(mt_EntityRelationship_Diagram);
+			insertFileNew();
+			insertStdItems();
+			insertSeparator();
+			insertStdItem(mt_Expand_All);
+			insertStdItem(mt_Collapse_All);
+			break;
+
 		case mt_UseCase_Diagram:
 		case mt_Sequence_Diagram:
 		case mt_Class_Diagram:
@@ -823,11 +885,17 @@ void ListPopupMenu::setupMenu(Menu_Type type, UMLView* view) {
 		case mt_Activity_Diagram:
 		case mt_Component_Diagram:
 		case mt_Deployment_Diagram:
-			insertStdItems(false);
+		case mt_EntityRelationship_Diagram:
+			//don't insert standard items because cut/copy not currently
+			// possible with tabbed diagrams (it didn't work anyway)
+			//insertStdItems(false);
+			insertStdItem(mt_Rename);
+			insertStdItem(mt_Delete);
 			insertStdItem(mt_Export_Image);
 			insertStdItem(mt_Properties);
 			break;
 
+		//FIXME a lot of these insertItem()s could be insertStandardItem()s
 		case mt_On_UseCase_Diagram:
 			m_pInsert = new KPopupMenu( this, "New" );
 			m_pInsert -> insertItem(m_pixmap[pm_Actor], i18n( "Actor..." ), mt_Actor );
@@ -891,6 +959,14 @@ void ListPopupMenu::setupMenu(Menu_Type type, UMLView* view) {
 			setupDiagramMenu(view);
 			break;
 
+		case mt_On_EntityRelationship_Diagram:
+			m_pInsert = new KPopupMenu(this, "New");
+			m_pInsert->insertItem(m_pixmap[pm_Entity], i18n("Entity..."), mt_Entity);
+			insertFileNew();
+			insertSeparator();
+			setupDiagramMenu(view);
+			break;
+
 		case mt_On_Sequence_Diagram:
 		case mt_On_Collaboration_Diagram:
 			m_pInsert = new KPopupMenu( this, "New" );
@@ -905,6 +981,7 @@ void ListPopupMenu::setupMenu(Menu_Type type, UMLView* view) {
 			m_pInsert = new KPopupMenu(this,"New");
 			m_pInsert -> insertItem(SmallIcon( "CVpublic_var"), i18n("Attribute"), mt_Attribute);
 			m_pInsert -> insertItem(SmallIcon( "CVpublic_meth"), i18n("Operation"), mt_Operation);
+			m_pInsert -> insertItem(SmallIcon("source"), i18n("Template"), mt_Template);
 			insertFileNew();
 			insertStdItems();
 			insertStdItem(mt_Properties);
@@ -913,6 +990,7 @@ void ListPopupMenu::setupMenu(Menu_Type type, UMLView* view) {
 		case mt_Interface:
 			m_pInsert = new KPopupMenu(this,"New");
 			m_pInsert->insertItem(SmallIcon("CVpublic_meth"), i18n("Operation"), mt_Operation);
+			m_pInsert -> insertItem(SmallIcon("source"), i18n("Template"), mt_Template);
 			insertFileNew();
 			insertStdItems();
 			insertStdItem(mt_Properties);
@@ -937,6 +1015,7 @@ void ListPopupMenu::setupMenu(Menu_Type type, UMLView* view) {
 		case mt_UseCase:
 		case mt_Attribute:
 		case mt_Operation:
+		case mt_Template:
 			insertStdItems(false);
 			insertStdItem(mt_Properties);
 			break;
@@ -961,6 +1040,10 @@ void ListPopupMenu::setupMenu(Menu_Type type, UMLView* view) {
 			insertStdItem(mt_New_EnumLiteral);
 			break;
 
+		case mt_New_EntityAttribute:
+			insertStdItem(mt_New_EntityAttribute);
+			break;
+
 		case mt_New_Activity:
 			m_pInsert = new KPopupMenu(this,"New");
 			insertStdItem(mt_New_Activity);
@@ -1006,6 +1089,12 @@ void ListPopupMenu::setupMenu(Menu_Type type, UMLView* view) {
 			insertStdItem(mt_Properties);
 			break;
 
+		case mt_EntityAttribute_Selected:
+			insertStdItem(mt_New_EntityAttribute);
+			insertStdItem(mt_Delete);
+			insertStdItem(mt_Properties);
+			break;
+
 		case mt_Association_Selected:
 			insertStdItem(mt_Delete);
 			insertStdItem(mt_Properties);
@@ -1015,32 +1104,28 @@ void ListPopupMenu::setupMenu(Menu_Type type, UMLView* view) {
 			insertItem(SmallIcon( "editdelete"),i18n("Delete Anchor"), mt_Delete);
 			break;
 
+		case mt_RoleNameA:
+			insertAssocItem(i18n("Change Role A Name..."), mt_Rename_RoleAName);
+			break;
+
+		case mt_RoleNameB:
+			insertAssocItem(i18n("Change Role B Name..."), mt_Rename_RoleBName);
+			break;
+
 		case mt_MultiB:
-			insertStdItem(mt_Delete_Association);
-			insertItem(i18n("Change Multiplicity..."), mt_Rename_MultiB);
-			insertStdItem(mt_Change_Font);
-			insertStdItem(mt_Reset_Label_Positions);
-			insertStdItem(mt_Properties);
+			insertAssocItem(i18n("Change Multiplicity..."), mt_Rename_MultiB);
 			break;
 
 		case mt_MultiA:
-			insertStdItem(mt_Delete_Association);
-			insertItem(i18n("Change Multiplicity..."), mt_Rename_MultiA);
-			insertStdItem(mt_Change_Font);
-			insertStdItem(mt_Reset_Label_Positions);
-			insertStdItem(mt_Properties);
+			insertAssocItem(i18n("Change Multiplicity..."), mt_Rename_MultiA);
 			break;
 
 		case mt_Name:
-			insertStdItem(mt_Delete_Association);
-			insertItem(i18n("Change Name"), mt_Rename_Name);
-			insertItem(SmallIcon( "fonts"), i18n( "Change Font" ), mt_Change_Font );
-			insertStdItem(mt_Reset_Label_Positions);
-			insertStdItem(mt_Properties);
+			insertAssocItem(i18n("Change Name"), mt_Rename_Name);
 			break;
 
-		case mt_RoleName:
-			insertStdItem(mt_Delete_Association);
+		case mt_FullAssociation:
+			insertStdItem(mt_Delete);
 			insertItem(i18n("Change Association Name..."), mt_Rename_Name);
 			insertItem(i18n("Change Role A Name..."), mt_Rename_RoleAName);
 			insertItem(i18n("Change Role B Name..."), mt_Rename_RoleBName);
@@ -1056,7 +1141,7 @@ void ListPopupMenu::setupMenu(Menu_Type type, UMLView* view) {
 //			insertSeparator();
 			insertStdItem(mt_Delete);
 			insertStdItem(mt_Change_Font);
-			insertStdItem(mt_Operation);
+			insertStdItem(mt_New_Operation);
 			insertItem(i18n("Select Operation..."), mt_Select_Operation);
 			break;
 		default:
