@@ -1,3 +1,8 @@
+ /*
+  *  copyright (C) 2002-2004
+  *  Umbrello UML Modeller Authors <uml-devel@ uml.sf.net>
+  */
+
 /***************************************************************************
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -18,7 +23,6 @@
 #include "floatingtext.h"
 #include "objectwidget.h"
 #include "classifier.h"
-#include "dialogs/selectopdlg.h"
 #include "umlview.h"
 #include "umldoc.h"
 #include "uml.h"
@@ -26,45 +30,53 @@
 
 MessageWidget::MessageWidget(UMLView * view, ObjectWidget* a, ObjectWidget* b,
 			     FloatingText* ft, int y,
-			     Sequence_Message_Type sequenceMessageType, int id)
+			     Uml::Sequence_Message_Type sequenceMessageType,
+			     Uml::IDType id /* = Uml::id_None */)
   : UMLWidget(view, id) {
 	init();
-	m_pOw[A] = a;
-	m_pOw[B] = b;
+	m_pOw[Uml::A] = a;
+	m_pOw[Uml::B] = b;
 	m_pFText = ft;
 	m_nY = y;
 	m_sequenceMessageType = sequenceMessageType;
+	if (m_sequenceMessageType == Uml::sequence_message_creation) {
+		y -= m_pOw[Uml::B]->getHeight() / 2;
+		m_pOw[Uml::B]->setY(y);
+	}
 
 	//CHECK: This is contorted - it should be in the caller's responsibility:
-	ft->setUMLObject(b->getUMLObject());
-	ft -> setLink(this);
+	if (ft) {
+		ft->setUMLObject(b->getUMLObject());
+		setLinkAndTextPos();
+	}
 
-	connect(m_pOw[A], SIGNAL(sigWidgetMoved(int)), this, SLOT(slotWidgetMoved(int)));
-	connect(m_pOw[B], SIGNAL(sigWidgetMoved(int)), this, SLOT(slotWidgetMoved(int)));
+	connect(m_pOw[Uml::A], SIGNAL(sigWidgetMoved(Uml::IDType)), this, SLOT(slotWidgetMoved(Uml::IDType)));
+	connect(m_pOw[Uml::B], SIGNAL(sigWidgetMoved(Uml::IDType)), this, SLOT(slotWidgetMoved(Uml::IDType)));
 	calculateWidget();
 	y = y < getMinHeight() ? getMinHeight() : y;
 	y = y > getMaxHeight() ? getMaxHeight() : y;
 	m_nY = y;
 
-	connect(this, SIGNAL(sigMessageMoved()), m_pOw[A], SLOT(slotMessageMoved()) );
-	connect(this, SIGNAL(sigMessageMoved()), m_pOw[B], SLOT(slotMessageMoved()) );
-	m_pOw[A] -> messageAdded(this);
-	m_pOw[B] -> messageAdded(this);
+	connect(this, SIGNAL(sigMessageMoved()), m_pOw[Uml::A], SLOT(slotMessageMoved()) );
+	connect(this, SIGNAL(sigMessageMoved()), m_pOw[Uml::B], SLOT(slotMessageMoved()) );
+	m_pOw[Uml::A] -> messageAdded(this);
+	m_pOw[Uml::B] -> messageAdded(this);
 }
 
-MessageWidget::MessageWidget(UMLView * view, Sequence_Message_Type sequenceMessageType, int id)
+MessageWidget::MessageWidget(UMLView * view, Uml::Sequence_Message_Type seqMsgType, Uml::IDType id)
   : UMLWidget(view, id) {
 	init();
-	m_sequenceMessageType = sequenceMessageType;
+	m_sequenceMessageType = seqMsgType;
 }
 
 void MessageWidget::init() {
-	UMLWidget::setBaseType(wt_Message);
+	UMLWidget::setBaseType(Uml::wt_Message);
 	m_bIgnoreSnapToGrid = true;
 	m_bIgnoreSnapComponentSizeToGrid = true;
-	m_pOw[A] = m_pOw[B] = NULL;
+	m_pOw[Uml::A] = m_pOw[Uml::B] = NULL;
 	m_pFText = NULL;
 	m_nY = 0;
+	m_inSelection = false;
 	setVisible(true);
 }
 
@@ -72,28 +84,30 @@ MessageWidget::~MessageWidget() {
 }
 
 void MessageWidget::draw(QPainter& p, int offsetX, int offsetY) {
-	if(!m_pOw[A] || !m_pOw[B]) {
+	if(!m_pOw[Uml::A] || !m_pOw[Uml::B]) {
 		return;
 	}
 	UMLWidget::draw(p, offsetX, offsetY);
-	if (m_sequenceMessageType == sequence_message_synchronous) {
+	if (m_sequenceMessageType == Uml::sequence_message_synchronous) {
 		drawSynchronous(p, offsetX, offsetY);
-	} else if (m_sequenceMessageType == sequence_message_asynchronous) {
+	} else if (m_sequenceMessageType == Uml::sequence_message_asynchronous) {
 		drawAsynchronous(p, offsetX, offsetY);
+	} else if (m_sequenceMessageType == Uml::sequence_message_creation) {
+		drawCreation(p, offsetX, offsetY);
 	} else {
 		kdWarning() << "Unknown message type" << endl;
 	}
 }
 
 void MessageWidget::drawSynchronous(QPainter& p, int offsetX, int offsetY) {
-	int x1 = m_pOw[A]->getX();
-	int x2 = m_pOw[B]->getX();
+	int x1 = m_pOw[Uml::A]->getX();
+	int x2 = m_pOw[Uml::B]->getX();
 	int w = getWidth() - 1;
 	int h = getHeight();
 
-	bool messageOverlaps = m_pOw[A] -> messageOverlap( getY(), this );
+	bool messageOverlaps = m_pOw[Uml::A] -> messageOverlap( getY(), this );
 
-	if(m_pOw[A] == m_pOw[B]) {
+	if(m_pOw[Uml::A] == m_pOw[Uml::B]) {
 		p.fillRect( offsetX, offsetY, 17, h,  QBrush(white) );			//box
 		p.drawRect(offsetX, offsetY, 17, h);					//box
 		p.drawLine(offsetX + 17, offsetY + 3, offsetX + w, offsetY + 3);	//arrow body lines
@@ -155,14 +169,14 @@ void MessageWidget::drawSynchronous(QPainter& p, int offsetX, int offsetY) {
 }
 
 void MessageWidget::drawAsynchronous(QPainter& p, int offsetX, int offsetY) {
-	int x1 = m_pOw[A]->getX();
-	int x2 = m_pOw[B]->getX();
+	int x1 = m_pOw[Uml::A]->getX();
+	int x2 = m_pOw[Uml::B]->getX();
 	int w = getWidth() - 1;
 	int h = getHeight() - 1;
-	bool messageOverlapsA = m_pOw[A] -> messageOverlap( getY(), this );
-	//bool messageOverlapsB = m_pOw[B] -> messageOverlap( getY(), this );
+	bool messageOverlapsA = m_pOw[Uml::A] -> messageOverlap( getY(), this );
+	//bool messageOverlapsB = m_pOw[Uml::B] -> messageOverlap( getY(), this );
 
-	if(m_pOw[A] == m_pOw[B]) {
+	if(m_pOw[Uml::A] == m_pOw[Uml::B]) {
 		if (messageOverlapsA)  {
 			offsetX += 7;
 			w -= 7;
@@ -195,13 +209,69 @@ void MessageWidget::drawAsynchronous(QPainter& p, int offsetX, int offsetY) {
 		p.drawLine(offsetX, offsetY + 4, offsetX + 4, offsetY + 7);
 	}
 
-	if (m_bSelected && m_pOw[A] == m_pOw[B]) {
+	if (m_bSelected && m_pOw[Uml::A] == m_pOw[Uml::B]) {
 		drawSelected(&p, offsetX, offsetY, true);
 	} else if (m_bSelected) {
 		drawSelected(&p, offsetX, offsetY);
 	}
 }
 
+void MessageWidget::drawCreation(QPainter& p, int offsetX, int offsetY) {
+	int x1 = m_pOw[Uml::A]->getX();
+	int x2 = m_pOw[Uml::B]->getX();
+	int w = getWidth() - 1;
+	//int h = getHeight() - 1;
+	bool messageOverlapsA = m_pOw[Uml::A] -> messageOverlap( getY(), this );
+	//bool messageOverlapsB = m_pOw[Uml::B] -> messageOverlap( getY(), this );
+
+	if (x1 < x2) {
+		if (messageOverlapsA)  {
+			offsetX += 7;
+			w -= 7;
+		}
+		p.drawLine(offsetX, offsetY + 4, offsetX + w, offsetY + 4);
+		p.drawLine(offsetX + w, offsetY + 4, offsetX + w - 4, offsetY + 1);
+		p.drawLine(offsetX + w, offsetY + 4, offsetX + w - 4, offsetY + 7);
+		if (messageOverlapsA)  {
+			offsetX -= 7;
+		}
+	} else	{
+		if (messageOverlapsA)  {
+			w -= 7;
+		}
+		p.drawLine(offsetX, offsetY + 4, offsetX + w, offsetY + 4);
+		p.drawLine(offsetX, offsetY + 4, offsetX + 4, offsetY + 1);
+		p.drawLine(offsetX, offsetY + 4, offsetX + 4, offsetY + 7);
+	}
+
+	if (m_bSelected && m_pOw[Uml::A] == m_pOw[Uml::B]) {
+		drawSelected(&p, offsetX, offsetY, true);
+	} else if (m_bSelected) {
+		drawSelected(&p, offsetX, offsetY);
+	}
+}
+
+bool MessageWidget::onWidget(const QPoint & p) {
+	if (m_sequenceMessageType == Uml::sequence_message_asynchronous) {
+		return UMLWidget::onWidget(p);
+	}
+	// Synchronous message:
+	// Consists of top arrow (call) and bottom arrow (return.)
+	if (p.x() < getX() || p.x() > getX() + getWidth())
+		return false;
+	const int tolerance = 4;  // pixels
+	const int pY = p.y();
+	const int topArrowY = getY();
+	const int bottomArrowY = topArrowY + getHeight();
+	if (pY < topArrowY - tolerance || pY > bottomArrowY + tolerance)
+		return false;
+	if (getHeight() <= 2 * tolerance)
+		return true;
+	if (pY > topArrowY + tolerance && pY < bottomArrowY - tolerance)
+		return false;
+	return true;
+}
+
 void MessageWidget::setTextPosition() {
 	if (m_pFText == NULL) {
 		kdDebug() << "MessageWidget::setTextPosition: m_pFText is NULL"
@@ -211,38 +281,50 @@ void MessageWidget::setTextPosition() {
 	if (m_pFText->getText().isEmpty())
 		return;
 	m_pFText->calculateSize();
-	const int xLowerBound = getX() + 5;
-	int ftX = m_pFText->getX();
-	if (ftX < xLowerBound || m_pFText->getRole() == tr_Seq_Message_Self)
-		ftX = xLowerBound;
-	else {
-		const int objB_seqLineX = m_pOw[B]->getX() + m_pOw[B]->getWidth() / 2;
-		const int xUpperBound = objB_seqLineX - m_pFText->getWidth() - 5;
-		if (xUpperBound <= xLowerBound)
-			ftX = xLowerBound;
-		else if (ftX > xUpperBound)
-			ftX = xUpperBound;
+	int ftX = constrainX(m_pFText->getX(), m_pFText->getWidth(), m_pFText->getRole());
+	int ftY = getY() - m_pFText->getHeight();
+	m_pFText->setX( ftX );
+	m_pFText->setY( ftY );
+}
+
+int MessageWidget::constrainX(int textX, int textWidth, Uml::Text_Role tr) {
+	int result = textX;
+	const int minTextX = getX() + 5;
+	if (textX < minTextX || tr == Uml::tr_Seq_Message_Self) {
+		result = minTextX;
+	} else {
+		const int objB_seqLineX = m_pOw[Uml::B]->getX() + m_pOw[Uml::B]->getWidth() / 2;
+		const int maxTextX = objB_seqLineX - textWidth - 5;
+		if (maxTextX <= minTextX)
+			result = minTextX;
+		else if (textX > maxTextX)
+			result = maxTextX;
 	}
-	m_pFText->setLinePos(ftX, getY() - m_pFText->getHeight());
+	return result;
 }
 
-void MessageWidget::updateMessagePos(int textHeight, int& newX, int& newY) {
-	newX = getX() + 5;
-	int minHeight = getMinHeight();
-	if (newY < minHeight)
-		newY = minHeight;
-	int maxHeight = getMaxHeight() - textHeight - 5;
-	if (newY > maxHeight)
-		newY = maxHeight;
-	setX( newX - 5 );  // what good is this? coordinate remains the same
-	setY( newY + textHeight );
+void MessageWidget::constrainTextPos(int &textX, int &textY, int textWidth, int textHeight,
+				     Uml::Text_Role tr) {
+	textX = constrainX(textX, textWidth, tr);
+	// Constrain Y.
+	const int minTextY = getMinHeight();
+	const int maxTextY = getMaxHeight() - textHeight - 5;
+	if (textY < minTextY)
+		textY = minTextY;
+	else if (textY > maxTextY)
+		textY = maxTextY;
+	setY( textY + textHeight );   // NB: side effect
 }
 
-void MessageWidget::setupAfterFTsetLink(FloatingText* /*ft*/) {
+void MessageWidget::setLinkAndTextPos() {
+	if (m_pFText == NULL)
+		return;
+	m_pFText->setLink(this);
 	setTextPosition();
 }
 
 void MessageWidget::moveEvent(QMoveEvent* /*m*/) {
+	//kdDebug() << "MessageWidget::moveEvent: m_pFText is " << m_pFText << endl;
 	if (!m_pFText) {
 		return;
 	}
@@ -266,8 +348,8 @@ void MessageWidget::calculateWidget() {
 	setY(m_nY);
 }
 
-void MessageWidget::slotWidgetMoved(int id) {
-	if(m_pOw[A] -> getLocalID() == id || m_pOw[B] -> getLocalID() == id) {
+void MessageWidget::slotWidgetMoved(Uml::IDType id) {
+	if(m_pOw[Uml::A] -> getLocalID() == id || m_pOw[Uml::B] -> getLocalID() == id) {
 		m_nY = getY();
 		m_nY = m_nY < getMinHeight()?getMinHeight():m_nY;
 		m_nY = m_nY > getMaxHeight()?getMaxHeight():m_nY;
@@ -281,7 +363,7 @@ void MessageWidget::slotWidgetMoved(int id) {
 }
 
 bool MessageWidget::contains(ObjectWidget * w) {
-	if(m_pOw[A] == w || m_pOw[B] == w)
+	if(m_pOw[Uml::A] == w || m_pOw[Uml::B] == w)
 		return true;
 	else
 		return false;
@@ -291,12 +373,25 @@ void MessageWidget::slotMenuSelection(int sel) {
 	if(sel == ListPopupMenu::mt_Delete) {
 		// This will clean up this widget and the text widget:
 		m_pView -> removeWidget(this);
-	} else
+	} else {
+		if (m_pFText == NULL) {
+			Uml::Text_Role tr = Uml::tr_Seq_Message;
+			if (m_pOw[Uml::A] == m_pOw[Uml::B])
+				tr = Uml::tr_Seq_Message_Self;
+			m_pFText = new FloatingText( m_pView, tr );
+			m_pFText->setFont(UMLWidget::getFont());
+			setLinkAndTextPos();
+			m_pFText->setText("");
+			m_pFText->setActivated();
+			m_pFText->setUMLObject( m_pOw[Uml::B]->getUMLObject() );
+		}
 		m_pFText -> slotMenuSelection(sel);
+	}
 }
 
 void MessageWidget::mouseDoubleClickEvent(QMouseEvent * /*me*/) {
-	if(m_pView -> getCurrentCursor() == WorkToolBar::tbb_Arrow)
+	if (m_pView->getCurrentCursor() == WorkToolBar::tbb_Arrow &&
+	    m_pFText != NULL)
 		m_pFText -> slotMenuSelection(ListPopupMenu::mt_Select_Operation);
 }
 
@@ -304,30 +399,25 @@ bool MessageWidget::activate(IDChangeLog * Log /*= 0*/) {
 	bool status = UMLWidget::activate(Log);
 	if(!status)
 		return false;
-	if (m_pOw[A] == NULL || m_pOw[B] == NULL) {
+	if (m_pOw[Uml::A] == NULL || m_pOw[Uml::B] == NULL) {
 		kdDebug() << "MessageWidget::activate: can't make message" << endl;
 		return false;
 	}
 	if( !m_pFText ) {
-		Text_Role tr = tr_Seq_Message;
-		if (m_pOw[A] ==m_pOw[B])
-			tr = tr_Seq_Message_Self;
+		Uml::Text_Role tr = Uml::tr_Seq_Message;
+		if (m_pOw[Uml::A] == m_pOw[Uml::B])
+			tr = Uml::tr_Seq_Message_Self;
 		m_pFText = new FloatingText( m_pView, tr, "" );
 		m_pFText->setFont(UMLWidget::getFont());
-	} else if (m_pFText->getID() == -1) {
-		int newid = UMLApp::app()->getDocument()->getUniqueID();
-		kdDebug() << "MessageWidget::activate: assigning new id=" << newid
-			  << " to m_pFText" << endl;
-		m_pFText -> setID(newid);
 	}
-	m_pFText -> setLink(this);
+	setLinkAndTextPos();
 	m_pFText -> setText("");
 	m_pFText->setActivated();
 	QString messageText = m_pFText->getText();
 	m_pFText->setVisible( messageText.length() > 1 );
-	m_pFText -> setUMLObject( m_pOw[B] -> getUMLObject() );
-	connect(m_pOw[A], SIGNAL(sigWidgetMoved(int)), this, SLOT(slotWidgetMoved(int)));
-	connect(m_pOw[B], SIGNAL(sigWidgetMoved(int)), this, SLOT(slotWidgetMoved(int)));
+	m_pFText -> setUMLObject( m_pOw[Uml::B] -> getUMLObject() );
+	connect(m_pOw[Uml::A], SIGNAL(sigWidgetMoved(Uml::IDType)), this, SLOT(slotWidgetMoved(Uml::IDType)));
+	connect(m_pOw[Uml::B], SIGNAL(sigWidgetMoved(Uml::IDType)), this, SLOT(slotWidgetMoved(Uml::IDType)));
 	if ( ! UMLApp::app()->getDocument()->loading() )
 	{ // calculate the dimensions only if no XMI file is loaded
 		// - this functions derives the dimension properties of this widget ( as said by the
@@ -337,10 +427,10 @@ bool MessageWidget::activate(IDChangeLog * Log /*= 0*/) {
 		calculateDimensions();
 	}
 
-	connect(this, SIGNAL(sigMessageMoved()), m_pOw[A], SLOT(slotMessageMoved()) );
-	connect(this, SIGNAL(sigMessageMoved()), m_pOw[B], SLOT(slotMessageMoved()) );
-	m_pOw[A] -> messageAdded(this);
-	m_pOw[B] -> messageAdded(this);
+	connect(this, SIGNAL(sigMessageMoved()), m_pOw[Uml::A], SLOT(slotMessageMoved()) );
+	connect(this, SIGNAL(sigMessageMoved()), m_pOw[Uml::B], SLOT(slotMessageMoved()) );
+	m_pOw[Uml::A] -> messageAdded(this);
+	m_pOw[Uml::B] -> messageAdded(this);
 
 	emit sigMessageMoved();
 	return status;
@@ -352,17 +442,17 @@ void MessageWidget::setMessageText(FloatingText *ft) {
 	setTextPosition();
 }
 
-void MessageWidget::setText(FloatingText *ft, QString newText) {
+void MessageWidget::setText(FloatingText *ft, const QString &newText) {
 	ft->setText(newText);
 	UMLApp::app()->getDocument()->setModified(true);
 }
 
-void MessageWidget::setSeqNumAndOp(QString seqNum, QString op) {
+void MessageWidget::setSeqNumAndOp(const QString &seqNum, const QString &op) {
 	setSequenceNumber( seqNum );
 	setOperation( op );
 }
 
-void MessageWidget::setSequenceNumber( QString sequenceNumber ) {
+void MessageWidget::setSequenceNumber( const QString &sequenceNumber ) {
 	m_SequenceNumber = sequenceNumber;
 }
 
@@ -370,7 +460,7 @@ QString MessageWidget::getSequenceNumber() const {
 	return m_SequenceNumber;
 }
 
-void MessageWidget::setOperation( QString operation ) {
+void MessageWidget::setOperation( const QString &operation ) {
 	m_Operation = operation;
 }
 
@@ -378,6 +468,10 @@ QString MessageWidget::getOperation() const {
 	return m_Operation;
 }
 
+void MessageWidget::lwSetFont (QFont font) {
+	UMLWidget::setFont( font );
+}
+
 UMLClassifier *MessageWidget::getOperationOwner(FloatingText *ft) {
 	UMLObject *pObject = ft->getUMLObject();
 	if (pObject == NULL)
@@ -386,7 +480,7 @@ UMLClassifier *MessageWidget::getOperationOwner(FloatingText *ft) {
 	return c;
 }
 
-void MessageWidget::setOperationText(FloatingText *ft, QString opText) {
+void MessageWidget::setOperationText(FloatingText *ft, const QString &opText) {
 	setOperation(opText);
 	ft->setMessageText();
 }
@@ -401,10 +495,12 @@ UMLClassifier * MessageWidget::getSeqNumAndOp(FloatingText *ft, QString& seqNum,
 }
 
 void MessageWidget::calculateDimensions() {
-	if (m_sequenceMessageType == sequence_message_synchronous) {
+	if (m_sequenceMessageType == Uml::sequence_message_synchronous) {
 		calculateDimensionsSynchronous();
-	} else if (m_sequenceMessageType == sequence_message_asynchronous) {
+	} else if (m_sequenceMessageType == Uml::sequence_message_asynchronous) {
 		calculateDimensionsAsynchronous();
+	} else if (m_sequenceMessageType == Uml::sequence_message_creation) {
+		calculateDimensionsCreation();
 	} else {
 		kdWarning() << "Unknown message type" << endl;
 	}
@@ -413,16 +509,16 @@ void MessageWidget::calculateDimensions() {
 void MessageWidget::calculateDimensionsSynchronous() {
 	int x = 0;
 
-	int x1 = m_pOw[A]->getX();
-	int x2 = m_pOw[B]->getX();
-	int w1 = m_pOw[A]->getWidth() / 2;
-	int w2 =  m_pOw[B]->getWidth() / 2;
+	int x1 = m_pOw[Uml::A]->getX();
+	int x2 = m_pOw[Uml::B]->getX();
+	int w1 = m_pOw[Uml::A]->getWidth() / 2;
+	int w2 =  m_pOw[Uml::B]->getWidth() / 2;
 	x1 += w1;
 	x2 += w2;
 
 	int widgetWidth = 0;
 	int widgetHeight = 0;
-	if( m_pOw[A] == m_pOw[B] ) {
+	if( m_pOw[Uml::A] == m_pOw[Uml::B] ) {
 		widgetWidth = 50;
 		x = x1 - 2;
 	} else if( x1 < x2 ) {
@@ -441,22 +537,22 @@ void MessageWidget::calculateDimensionsSynchronous() {
 
 	m_nPosX = x;
 	setSize(widgetWidth, widgetHeight);
-	adjustAssocs( this->getX(), this->getY() );//adjust assoc lines
+	adjustAssocs( getX(), getY() );//adjust assoc lines
 }
 
 void MessageWidget::calculateDimensionsAsynchronous() {
 	int x = 0;
 
-	int x1 = m_pOw[A]->getX();
-	int x2 = m_pOw[B]->getX();
-	int w1 = m_pOw[A] -> getWidth() / 2;
-	int w2 =  m_pOw[B] -> getWidth() / 2;
+	int x1 = m_pOw[Uml::A]->getX();
+	int x2 = m_pOw[Uml::B]->getX();
+	int w1 = m_pOw[Uml::A] -> getWidth() / 2;
+	int w2 =  m_pOw[Uml::B] -> getWidth() / 2;
 	x1 += w1;
 	x2 += w2;
 
 	int widgetWidth = 0;
 	int widgetHeight = 8;
-	if( m_pOw[A] == m_pOw[B] ) {
+	if( m_pOw[Uml::A] == m_pOw[Uml::B] ) {
 		widgetWidth = 50;
 		x = x1;
 		if( height() < 20 ) {
@@ -475,17 +571,44 @@ void MessageWidget::calculateDimensionsAsynchronous() {
 	widgetWidth -= 2;
 	m_nPosX = x;
 	setSize(widgetWidth, widgetHeight);
-	adjustAssocs( this->getX(), this->getY() );//adjust assoc lines
+	adjustAssocs( getX(), getY() );//adjust assoc lines
+}
+
+void MessageWidget::calculateDimensionsCreation() {
+	int x = 0;
+
+	int x1 = m_pOw[Uml::A]->getX();
+	int x2 = m_pOw[Uml::B]->getX();
+	int w1 = m_pOw[Uml::A] -> getWidth() / 2;
+	int w2 =  m_pOw[Uml::B] -> getWidth();
+	x1 += w1;
+	if (x1 > x2)
+		x2 += w2;
+
+	int widgetWidth = 0;
+	int widgetHeight = 8;
+	if ( x1 < x2 ) {
+		x = x1;
+		widgetWidth = x2 - x1;
+	} else {
+		x = x2;
+		widgetWidth = x1 - x2;
+	}
+	x += 1;
+	widgetWidth -= 2;
+	m_nPosX = x;
+	setSize(widgetWidth, widgetHeight);
+	adjustAssocs( getX(), getY() );//adjust assoc lines
 }
 
 void MessageWidget::cleanup() {
-	if (m_pOw[A]) {
-		disconnect(this, SIGNAL(sigMessageMoved()), m_pOw[A], SLOT(slotMessageMoved()) );
-		m_pOw[A]->messageRemoved(this);
+	if (m_pOw[Uml::A]) {
+		disconnect(this, SIGNAL(sigMessageMoved()), m_pOw[Uml::A], SLOT(slotMessageMoved()) );
+		m_pOw[Uml::A]->messageRemoved(this);
 	}
-	if (m_pOw[B]) {
-		disconnect(this, SIGNAL(sigMessageMoved()), m_pOw[B], SLOT(slotMessageMoved()) );
-		m_pOw[B]->messageRemoved(this);
+	if (m_pOw[Uml::B]) {
+		disconnect(this, SIGNAL(sigMessageMoved()), m_pOw[Uml::B], SLOT(slotMessageMoved()) );
+		m_pOw[Uml::B]->messageRemoved(this);
 	}
 
 	UMLWidget::cleanup();
@@ -499,8 +622,8 @@ void MessageWidget::mouseMoveEvent(QMouseEvent *me) {
 	int newX = 0, newY = 0;
 	int moveX, moveY;
 	if( m_bResizing ) {
-		int heightA = (int)((ObjectWidget*)m_pOw[A]) -> getEndLineY();
-		int heightB = (int)((ObjectWidget*)m_pOw[B]) -> getEndLineY();
+		int heightA = (int)((ObjectWidget*)m_pOw[Uml::A]) -> getEndLineY();
+		int heightB = (int)((ObjectWidget*)m_pOw[Uml::B]) -> getEndLineY();
 		int height = heightA;
 		if ( heightA > heightB )  {
 			height = heightB;
@@ -540,26 +663,27 @@ void MessageWidget::mouseMoveEvent(QMouseEvent *me) {
 	newY = getY() + moveY;
 	newY = newY < 0?0:newY;
 	if( count > 2 )
-		newY = this -> getY();  //only change y if not selected
-	newX = m_nPosX;
+		newY = getY();  //only change y if not selected
 	newY = newY < getMinHeight() ? getMinHeight() : newY;
 	newY = newY > getMaxHeight() ? getMaxHeight() : newY;
 
 	if (m_nOldX != newX || m_nOldY != newY) {
 		m_bMoved = true;
 	}
-	m_nOldX = newX;
 	m_nOldY = newY;
-	setX( newX );
 	setY( newY );
 	adjustAssocs(newX, newY);
+	if (m_sequenceMessageType == Uml::sequence_message_creation) {
+		const int objWidgetHalfHeight = m_pOw[Uml::B]->getHeight() / 2;
+		m_pOw[Uml::B]->UMLWidget::setY( newY - objWidgetHalfHeight );
+	}
 	moveEvent(0);
 }
 
 void MessageWidget::setSelected(bool _select) {
-	if( m_nOldID == -10 )//used to stop a recursive call
+	if( m_inSelection )//used to stop a recursive call
 	{
-		m_nOldID = 0;
+		m_inSelection = false;
 		return;
 	}
 	UMLWidget::setSelected( _select );
@@ -570,17 +694,18 @@ void MessageWidget::setSelected(bool _select) {
 	if( !m_bSelected && !m_pFText -> getSelected() )
 		return;
 
-	m_nOldID = -10;
+	m_inSelection = true;
 	m_pView -> setSelected( m_pFText, 0 );
 	m_pFText -> setSelected( m_bSelected );
 }
 
 int MessageWidget::getMinHeight() {
-	if( !m_pOw[A] || !m_pOw[B] ) {
+	if (!m_pOw[Uml::A] || !m_pOw[Uml::B] ||
+	    m_sequenceMessageType == Uml::sequence_message_creation) {
 		return 0;
 	}
-	int heightA = m_pOw[A]->getY() + m_pOw[A]->getHeight();
-	int heightB = m_pOw[B]->getY() + m_pOw[B]->getHeight();
+	int heightA = m_pOw[Uml::A]->getY() + m_pOw[Uml::A]->getHeight();
+	int heightB = m_pOw[Uml::B]->getY() + m_pOw[Uml::B]->getHeight();
 	int height = heightA;
 	if( heightA < heightB ) {
 		height = heightB;
@@ -589,11 +714,11 @@ int MessageWidget::getMinHeight() {
 }
 
 int MessageWidget::getMaxHeight() {
-	if( !m_pOw[A] || !m_pOw[B] ) {
+	if( !m_pOw[Uml::A] || !m_pOw[Uml::B] ) {
 		return 0;
 	}
-	int heightA = (int)((ObjectWidget*)m_pOw[A])->getEndLineY();
-	int heightB = (int)((ObjectWidget*)m_pOw[B])->getEndLineY();
+	int heightA = (int)((ObjectWidget*)m_pOw[Uml::A])->getEndLineY();
+	int heightB = (int)((ObjectWidget*)m_pOw[Uml::B])->getEndLineY();
 	int height = heightA;
 	if( heightA > heightB ) {
 		height = heightB;
@@ -607,7 +732,8 @@ void MessageWidget::mousePressEvent(QMouseEvent* me) {
 		return;
 	}
 	//resize only interests a message to self or an asynchronous message
-	if ( m_sequenceMessageType == sequence_message_asynchronous && m_pOw[A] != m_pOw[B] ) {
+	if ( m_sequenceMessageType == Uml::sequence_message_asynchronous &&
+	     m_pOw[Uml::A] != m_pOw[Uml::B] ) {
 		return;
 	}
 	int m = 10;
@@ -629,27 +755,28 @@ void MessageWidget::mouseReleaseEvent( QMouseEvent * me ) {
 	m_pView->setCursor( KCursor::arrowCursor() );
 }
 
-void MessageWidget::setWidget(ObjectWidget * ow, Role_Type role) {
+void MessageWidget::setWidget(ObjectWidget * ow, Uml::Role_Type role) {
 	m_pOw[role] = ow;
 }
 
-ObjectWidget* MessageWidget::getWidget(Role_Type role) {
+ObjectWidget* MessageWidget::getWidget(Uml::Role_Type role) {
 	return m_pOw[role];
 }
 
 void MessageWidget::saveToXMI( QDomDocument & qDoc, QDomElement & qElement ) {
 	QDomElement messageElement = qDoc.createElement( "messagewidget" );
 	UMLWidget::saveToXMI( qDoc, messageElement );
-	messageElement.setAttribute( "textid", m_pFText->getID() );
-	messageElement.setAttribute( "widgetaid", m_pOw[A]->getLocalID() );
-	messageElement.setAttribute( "widgetbid", m_pOw[B]->getLocalID() );
+	messageElement.setAttribute( "widgetaid", ID2STR(m_pOw[Uml::A]->getLocalID()) );
+	messageElement.setAttribute( "widgetbid", ID2STR(m_pOw[Uml::B]->getLocalID()) );
 	messageElement.setAttribute( "operation", m_Operation );
 	messageElement.setAttribute( "seqnum", m_SequenceNumber );
 	messageElement.setAttribute( "sequencemessagetype", m_sequenceMessageType );
 
 	// save the corresponding message text
-	if( m_pFText )
+	if (m_pFText && !m_pFText->getText().isEmpty()) {
+		messageElement.setAttribute( "textid", ID2STR(m_pFText->getID()) );
 		m_pFText -> saveToXMI( qDoc, messageElement );
+	}
 
 	qElement.appendChild( messageElement );
 }
@@ -664,44 +791,44 @@ bool MessageWidget::loadFromXMI(QDomElement& qElement) {
 	m_Operation = qElement.attribute( "operation", "" );
 	m_SequenceNumber = qElement.attribute( "seqnum", "" );
 	QString sequenceMessageType = qElement.attribute( "sequencemessagetype", "1001" );
-	m_sequenceMessageType = (Sequence_Message_Type)sequenceMessageType.toInt();
+	m_sequenceMessageType = (Uml::Sequence_Message_Type)sequenceMessageType.toInt();
 
-	int aId = widgetaid.toInt();
-	int bId = widgetbid.toInt();
+	Uml::IDType aId = STR2ID(widgetaid);
+	Uml::IDType bId = STR2ID(widgetbid);
 
 	UMLWidget *pWA = m_pView -> findWidget( aId );
 	if (pWA == NULL) {
-		kdDebug() << "MessageWidget::loadFromXMI: role A object " << aId
-			  << " not found" << endl;
+		kdDebug() << "MessageWidget::loadFromXMI: role A object "
+			  << ID2STR(aId) << " not found" << endl;
 		return false;
 	}
 	UMLWidget *pWB = m_pView -> findWidget( bId );
 	if (pWB == NULL) {
-		kdDebug() << "MessageWidget::loadFromXMI: role B object " << bId
-			  << " not found" << endl;
+		kdDebug() << "MessageWidget::loadFromXMI: role B object "
+			  << ID2STR(bId) << " not found" << endl;
 		return false;
 	}
-	m_pOw[A] = dynamic_cast<ObjectWidget*>(pWA);
-	if (m_pOw[A] == NULL) {
-		kdDebug() << "MessageWidget::loadFromXMI: role A widget " << aId
-			  << " is not an ObjectWidget" << endl;
+	m_pOw[Uml::A] = dynamic_cast<ObjectWidget*>(pWA);
+	if (m_pOw[Uml::A] == NULL) {
+		kdDebug() << "MessageWidget::loadFromXMI: role A widget "
+			  << ID2STR(aId) << " is not an ObjectWidget" << endl;
 		return false;
 	}
-	m_pOw[B] = dynamic_cast<ObjectWidget*>(pWB);
-	if (m_pOw[B] == NULL) {
-		kdDebug() << "MessageWidget::loadFromXMI: role B widget " << bId
-			  << " is not an ObjectWidget" << endl;
+	m_pOw[Uml::B] = dynamic_cast<ObjectWidget*>(pWB);
+	if (m_pOw[Uml::B] == NULL) {
+		kdDebug() << "MessageWidget::loadFromXMI: role B widget "
+			  << ID2STR(bId) << " is not an ObjectWidget" << endl;
 		return false;
 	}
 
-	int textId = textid.toInt();
-	if (textId > 0) {
+	Uml::IDType textId = STR2ID(textid);
+	if (textId != Uml::id_None) {
 		UMLWidget *flotext = m_pView -> findWidget( textId );
 		if (flotext != NULL) {
 			// This only happens when loading files produced by
 			// umbrello-1.3-beta2.
 			m_pFText = static_cast<FloatingText*>(flotext);
-			m_pFText->setLink(this);
+			setLinkAndTextPos();
 			return true;
 		}
 	} else {
@@ -709,42 +836,30 @@ bool MessageWidget::loadFromXMI(QDomElement& qElement) {
 		textId = UMLApp::app()->getDocument()->getUniqueID();
 	}
 
-	Text_Role tr = tr_Seq_Message;
-	if (m_pOw[A] == m_pOw[B])
-		tr = tr_Seq_Message_Self;
-	m_pFText = new FloatingText( m_pView, tr, m_Operation, textId );
-	bool textLoadSuccess = false;
+	Uml::Text_Role tr = Uml::tr_Seq_Message;
+	if (m_pOw[Uml::A] == m_pOw[Uml::B])
+		tr = Uml::tr_Seq_Message_Self;
 
 	//now load child elements
 	QDomNode node = qElement.firstChild();
 	QDomElement element = node.toElement();
-	while( !element.isNull() ) {
+	if ( !element.isNull() ) {
 		QString tag = element.tagName();
-		if (tag == "floatingtext"  ) {
+		if (tag == "floatingtext") {
+			m_pFText = new FloatingText( m_pView, tr, m_Operation, textId );
 			if( ! m_pFText->loadFromXMI(element) ) {
 				// Most likely cause: The FloatingText is empty.
-				node = element.nextSibling();
-				element = node.toElement();
-				continue;
-			}
-			else {
-				textLoadSuccess = true;
-				break; // no more to load
+				delete m_pFText;
+				m_pFText = NULL;
 			}
+		} else {
+			kdError() << "MessageWidget::loadFromXMI: unknown tag "
+				  << tag << endl;
 		}
 	}
 
-	if ( ! textLoadSuccess ) {
-		// set FloatingText from tag m_Operation
-		// m_pFText->setText(m_Operation);
-		m_pFText->setFont(UMLWidget::getFont());
-		m_pFText->setActivated();
-		m_pFText->setVisible( m_Operation.length() > 1 );
-		m_pFText -> setUMLObject( m_pOw[B] -> getUMLObject() );
-	}
-
 	// always need this
-	m_pFText->setLink(this);
+	setLinkAndTextPos();
 
 	return true;
 }
