@@ -1,3 +1,8 @@
+ /*
+  *  copyright (C) 2002-2004
+  *  Umbrello UML Modeller Authors <uml-devel@ uml.sf.net>
+  */
+
 /***************************************************************************
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -9,7 +14,7 @@
 
 #include "usecase.h"
 
-UMLUseCase::UMLUseCase(const QString & name, int id)
+UMLUseCase::UMLUseCase(const QString & name, Uml::IDType id)
   : UMLCanvasObject(name, id) {
 	init();
 }
@@ -17,7 +22,7 @@ UMLUseCase::UMLUseCase(const QString & name, int id)
 UMLUseCase::~UMLUseCase() {}
 
 void UMLUseCase::init() {
-	m_BaseType = ot_UseCase;
+	m_BaseType = Uml::ot_UseCase;
 }
 
 UMLObject* UMLUseCase::clone() const {
